/**
 *******************************************************************************
 * @file    tmpm440_esio.h
 * @brief   This file provides all the functions prototypes for ESIO driver.
 * @version V2.0.2.3
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __TMPM440_ESIO_H
#define __TMPM440_ESIO_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "TMPM440.h"
#include "tx04_common.h"
#ifdef __IAR_SYSTEMS_ICC__
#if 1 != __LITTLE_ENDIAN__
#pragma bitfields=disjoint_types
#endif
#endif

/* #define LITTLE_ENDIAN */

#if defined(__TMPM440_ESIO_H)

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @addtogroup ESIO
  * @{
  */

/** @defgroup ESIO_Exported_Types
  * @{
  */

#define IS_ESIO_UNIT(param)             (((param) == TSB_ESIO0) || \
                                         ((param) == TSB_ESIO1) || \
                                         ((param) == TSB_ESIO2))

#define ESIO_TX                         (uint8_t)0x00
#define ESIO_RX                         (uint8_t)0x01
#define IS_ESIO_TXRX(param)             ((param) <= ESIO_RX)

#define IS_ESIO_FIFOLEVELINT(param)     ((param) <= 8U)

/* for CR2 bit2, 4-7 : INT control */
#define ESIO_INT_PERR      (uint32_t)0x04       /* Parity error interrupt control */
#define ESIO_INT_RXEND     (uint32_t)0x10       /* Receive completation interrupt control */
#define ESIO_INT_RXFIFO    (uint32_t)0x20       /* Receive FIFO interrupt control */
#define ESIO_INT_TXEND     (uint32_t)0x40       /* Transmit completation interrupt control */
#define ESIO_INT_TXFIFO    (uint32_t)0x80       /* Transmit FIFO interrupt control */
#define ESIO_INT_ALL       (uint32_t)0xF4       /* All above interrupt control */
#define IS_ESIO_INTSRC(param)              (((param) & ESIO_INT_ALL) && \
                                            (!((param) & (uint32_t) (~ESIO_INT_ALL))))


    typedef enum {
        ESIO_MODE_SPI = 0U,
        ESIO_MODE_SIO = 1U
    } ESIO_Mode;
#define IS_ESIO_MODE(param)             ((param) <= ESIO_MODE_SIO)


    typedef enum {
        ESIO_TRMODE_TX = 1U,    /* Half duplex (Transmit) mode */
        ESIO_TRMODE_RX = 2U,    /* Half duplex (Receive) mode  */
        ESIO_TRMODE_TXRX = 3U   /* Full duplex mode */
    } ESIO_TransferMode;
#define IS_ESIO_TRANSFERMODE(param)     (((param)>= ESIO_TRMODE_TX) && \
                                         ((param) <= ESIO_TRMODE_TXRX))

    typedef enum {
        ESIO_CS0 = 0U,
        ESIO_CS1 = 1U
    } ESIO_CSx;                 /* Chip select pin selection */
#define IS_ESIO_CSX(param)              ((param) <= ESIO_CS1)

#define IS_ESIO_TRANSFERNUM(param)              ((param) >= 1U)

    typedef enum {
        ESIO_TXPIN_FINALBIT = 1U,       /* Keep as the final bit */
        ESIO_TXPIN_KEEPLOW = 2U,        /* Keep as low level output  */
        ESIO_TXPIN_KEEPHIGH = 3U        /* Keep as high level output */
    } ESIO_TxPinInIdle;         /* status of ESIOxTXD pin during idle cycle */
#define IS_ESIO_TXPININIDLE(param)     (((param) >= ESIO_TXPIN_FINALBIT) && \
                                         ((param) <= ESIO_TXPIN_KEEPHIGH))

    typedef enum {
        ESIO_PHIT0_DIVIDE_2 = 0U,       /* phiT0/2 */
        ESIO_PHIT0_DIVIDE_4 = 1U,       /* phiT0/4 , the maximum clock */
        ESIO_PHIT0_DIVIDE_8 = 2U,       /* phiT0/8 */
        ESIO_PHIT0_DIVIDE_16 = 3U,      /* phiT0/16 */
        ESIO_PHIT0_DIVIDE_32 = 4U,      /* phiT0/32 */
        ESIO_PHIT0_DIVIDE_64 = 5U,      /* phiT0/64 */
        ESIO_PHIT0_DIVIDE_128 = 6U,     /* phiT0/128 */
        ESIO_PHIT0_DIVIDE_256 = 7U,     /* phiT0/256 */
        ESIO_PHIT0_DIVIDE_512 = 8U,     /* phiT0/512 */
        ESIO_PHIT0_DIVIDE_1024 = 9U     /* phiT0/1024 */
    } ESIO_BaudClock;
#define IS_ESIO_BAUDCLOCK(p1,p2)            ( ( (p1) <= ESIO_PHIT0_DIVIDE_1024) && \
                                               (!(((p1) == ESIO_PHIT0_DIVIDE_2) && ((p2) ==1U))))
#define IS_ESIO_DIVIDER(param)                (((param) >= 1U) && ((param) <= 16U))



#define ESIO_LSB_FIRST	    (uint8_t)0x00
#define ESIO_MSB_FIRST	    (uint8_t)0x01

#define ESIO_CLKPOL_LOW	    (uint8_t)0x00
#define ESIO_CLKPOL_HIGH	(uint8_t)0x01

#define ESIO_CS_ACTIVE_LOW	(uint8_t)0x00
#define ESIO_CS_ACTIVE_HIGH	(uint8_t)0x01

#define ESIO_PARITY_EVEN	(uint8_t)0x00
#define ESIO_PARITY_ODD	    (uint8_t)0x01

    typedef struct {
        uint8_t DataDirection;  /* LSB or MSB first */
        uint8_t FrameLength;    /* 8 to 32 bit */
        uint8_t CycleBetweenFrames;     /* 0 to 15 (Interval cycle between frames in burst mode) */
        uint8_t NumberOfLINE;   /* 1 to 4  */
        uint8_t ClkPolority;    /* Level of serial clock during idle time */
        uint8_t ShortestIdleCycle;      /* 1 to 15 */
        uint8_t CS1ActiveLevel; /* low active or high active for pin CS1 */
        uint8_t CS0ActiveLevel; /* low active or high active for pin CS0 */
        uint8_t DelayCycleNum_CS_SCLK;  /* 1 to 16 ( Delay cycle from asserting ESIOxCS to output serial clock */
        uint8_t DelayCycleNum_Negate_CS;        /* 1 to 16 ( Delay cycle of negating ESIOxCS ) */
        FunctionalState HorizontalParityCheck;  /* ENABLE or DISABLE */
        uint8_t HorizontalParity;       /* ESIO_PARITY_ODD or ESIO_PARITY_ EVEN */
        FunctionalState VerticalParityCheck;    /* ENABLE or DISABLE */
        uint8_t VerticalParity; /* ESIO_PARITY_ODD or ESIO_PARITY_ EVEN */
    } ESIO_InitTypeDef;
#define IS_ESIO_DATADIRECTION(param)        ((param) <= ESIO_MSB_FIRST)
#define IS_ESIO_FRAMELENGTH(param)          (((param)>= 8U) && ((param) <= 32U))
#define IS_ESIO_CYCLEBETWEENFRAMES(param)   ((param) <= 15U)
#define IS_ESIO_NUMBEROFLINE(param)         (((param)>= 1U) && ((param) <= 4U))
#define IS_ESIO_CLKPOLORITY(param)          ((param) <= ESIO_CLKPOL_HIGH)
#define IS_ESIO_SHORTESTIDLECYCLE(param)    (((param)>= 1U) && ((param) <= 15U))
#define IS_ESIO_CSACTIVELEVEL(param)        ((param) <= ESIO_CS_ACTIVE_HIGH)
#define IS_ESIO_DELAYCYCLENUM(param)        (((param)>= 1U) && ((param) <= 16U))
#define IS_ESIO_PARITYTYPE(param)           ((param) <= ESIO_PARITY_ODD)
#define IS_ESIO_PARITYINSINGLE(p1,p2)       (!((((p1)&0xFFU) == 1U) && ((p2) == ENABLE)))



/**
  * @brief Union to store ESIO status and store ESIO result
  */
#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {             /* big endian */
        uint32_t All;
        struct {
            uint32_t ESIO_RegUsing:1;   /* bit31: '1': ESIO registers must not be modified. */
            uint32_t Reserved2:7;
            uint32_t TxRUN:1;   /* bit23: '1': Transmit shift is operating */
            uint32_t TxCompleted:1;     /* bit22: '1': Transmission completed */
            uint32_t TxFIFO_INT:1;      /* bit21: '1': Transmit FIFO interrupt generated */
            uint32_t TxFIFOEmpty:1;     /* bit20: '1': Transmit FIFO is empty  */
            uint32_t TxFIFOFillLevel:4; /* bit19--16 , Transmit FIFO fill level */
            uint32_t Reserved1:8;
            uint32_t RxRUN:1;   /* bit7: '1': Receive shift is operating */
            uint32_t RxCompleted:1;     /* bit6: '1': Reception completed */
            uint32_t RxFIFO_INT:1;      /* bit5: '1': Receive FIFO interrupt generated */
            uint32_t RxFIFOFull:1;      /* bit4: '1': Receive FIFO is full  */
            uint32_t RxFIFOFillLevel:4; /* bit0--3 , Receive FIFO fill level */
        } Bit;
    } ESIO_StatusFlag;


#else
    typedef union {             /* little endian */
        uint32_t All;
        struct {
            uint32_t RxFIFOFillLevel:4; /* bit0--3 , Receive FIFO fill level */
            uint32_t RxFIFOFull:1;      /* bit4: '1': Receive FIFO is full  */
            uint32_t RxFIFO_INT:1;      /* bit5: '1': Receive FIFO interrupt generated */
            uint32_t RxCompleted:1;     /* bit6: '1': Reception completed */
            uint32_t RxRUN:1;   /* bit7: '1': Receive shift is operating */
            uint32_t Reserved1:8;
            uint32_t TxFIFOFillLevel:4; /* bit19--16 , Transmit FIFO fill level */
            uint32_t TxFIFOEmpty:1;     /* bit20: '1': Transmit FIFO is empty  */
            uint32_t TxFIFO_INT:1;      /* bit21: '1': Transmit FIFO interrupt generated */
            uint32_t TxCompleted:1;     /* bit22: '1': Transmission completed */
            uint32_t TxRUN:1;   /* bit23: '1': Transmit shift is operating */
            uint32_t Reserved2:7;
            uint32_t ESIO_RegUsing:1;   /* bit31: '1': ESIO registers must not be modified. */
        } Bit;
    } ESIO_StatusFlag;

#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {             /* little endian */
        uint32_t All;
        struct {
            uint32_t RxFIFOFillLevel:4; /* bit0--3 , Receive FIFO fill level */
            uint32_t RxFIFOFull:1;      /* bit4: '1': Receive FIFO is full  */
            uint32_t RxFIFO_INT:1;      /* bit5: '1': Receive FIFO interrupt generated */
            uint32_t RxCompleted:1;     /* bit6: '1': Reception completed */
            uint32_t RxRUN:1;   /* bit7: '1': Receive shift is operating */
            uint32_t Reserved1:8;
            uint32_t TxFIFOFillLevel:4; /* bit19--16 , Transmit FIFO fill level */
            uint32_t TxFIFOEmpty:1;     /* bit20: '1': Transmit FIFO is empty  */
            uint32_t TxFIFO_INT:1;      /* bit21: '1': Transmit FIFO interrupt generated */
            uint32_t TxCompleted:1;     /* bit22: '1': Transmission completed */
            uint32_t TxRUN:1;   /* bit23: '1': Transmit shift is operating */
            uint32_t Reserved2:7;
            uint32_t ESIO_RegUsing:1;   /* bit31: '1': ESIO registers must not be modified. */
        } Bit;
    } ESIO_StatusFlag;

#endif



/**
  * @brief Union to store ESIO Parity Error number and H/V parity error info
  */
#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {             /* big endian */
        uint32_t All;
        struct {                /* memer LINEy_ParityErrNum large than 2 means Parity error is occurred twice or more times. */
            uint32_t Reserved1:22;
            uint32_t VerticalParityErr:1;       /* bit9: '1': Vertical parity error occurred.  */
            uint32_t HorizontalParityErr:1;     /* bit8: '1': Horizontal parity error occurred.  */
            uint32_t LINE3_ParityErrNum:2;      /* bit7--6 , LINE 3 parity error numbers */
            uint32_t LINE2_ParityErrNum:2;      /* bit5--4 , LINE 2 parity error numbers */
            uint32_t LINE1_ParityErrNum:2;      /* bit3--2 , LINE 1 parity error numbers */
            uint32_t LINE0_ParityErrNum:2;      /* bit0--1 , LINE 0 parity error numbers */
        } Bit;
    } ESIO_ParityErrNum;


#else
    typedef union {             /* little endian */
        uint32_t All;
        struct {                /* memer LINEy_ParityErrNum large than 2 means Parity error is occurred twice or more times. */
            uint32_t LINE0_ParityErrNum:2;      /* bit0--1 , LINE 0 parity error numbers */
            uint32_t LINE1_ParityErrNum:2;      /* bit3--2 , LINE 1 parity error numbers */
            uint32_t LINE2_ParityErrNum:2;      /* bit5--4 , LINE 2 parity error numbers */
            uint32_t LINE3_ParityErrNum:2;      /* bit7--6 , LINE 3 parity error numbers */
            uint32_t HorizontalParityErr:1;     /* bit8: '1': Horizontal parity error occurred.  */
            uint32_t VerticalParityErr:1;       /* bit9: '1': Vertical parity error occurred.  */
            uint32_t Reserved1:22;
        } Bit;
    } ESIO_ParityErrNum;

#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {             /* little endian */
        uint32_t All;
        struct {                /* memer LINEy_ParityErrNum large than 2 means Parity error is occurred twice or more times. */
            uint32_t LINE0_ParityErrNum:2;      /* bit0--1 , LINE 0 parity error numbers */
            uint32_t LINE1_ParityErrNum:2;      /* bit3--2 , LINE 1 parity error numbers */
            uint32_t LINE2_ParityErrNum:2;      /* bit5--4 , LINE 2 parity error numbers */
            uint32_t LINE3_ParityErrNum:2;      /* bit7--6 , LINE 3 parity error numbers */
            uint32_t HorizontalParityErr:1;     /* bit8: '1': Horizontal parity error occurred.  */
            uint32_t VerticalParityErr:1;       /* bit9: '1': Vertical parity error occurred.  */
            uint32_t Reserved1:22;
        } Bit;
    } ESIO_ParityErrNum;

#endif

    typedef enum {
        ESIO_LINE0 = 0U,
        ESIO_LINE1 = 1U,
        ESIO_LINE2 = 2U,
        ESIO_LINE3 = 3U
    } ESIO_LINEx;
#define IS_ESIO_LINEx(param)            ((param) <= ESIO_LINE3)




/**
  * @brief Union to store ESIO Vertical Parity Error Frame
  */
#if defined ( __CC_ARM )
#ifdef __BIG_ENDIAN
    typedef union {             /* big endian */
        uint32_t All;
        struct {
            uint32_t LINE3_VerticalParityErrFrame:8;    /* The number of frame which has first vertical parity error in LINE 3 */
            uint32_t LINE2_VerticalParityErrFrame:8;
            uint32_t LINE1_VerticalParityErrFrame:8;
            uint32_t LINE0_VerticalParityErrFrame:8;
        } Bit;
    } ESIO_VerticalParityErrFrame;

#else
    typedef union {             /* little endian */
        uint32_t All;
        struct {
            uint32_t LINE0_VerticalParityErrFrame:8;    /* The number of frame which has first vertical parity error in LINE 0 */
            uint32_t LINE1_VerticalParityErrFrame:8;
            uint32_t LINE2_VerticalParityErrFrame:8;
            uint32_t LINE3_VerticalParityErrFrame:8;
        } Bit;
    } ESIO_VerticalParityErrFrame;

#endif
#elif  defined(__IAR_SYSTEMS_ICC__)
    typedef union {             /* little endian */
        uint32_t All;
        struct {
            uint32_t LINE0_VerticalParityErrFrame:8;    /* The number of frame which has first vertical parity error in LINE 0 */
            uint32_t LINE1_VerticalParityErrFrame:8;
            uint32_t LINE2_VerticalParityErrFrame:8;
            uint32_t LINE3_VerticalParityErrFrame:8;
        } Bit;
    } ESIO_VerticalParityErrFrame;

#endif





/** @} */
/* End of group ESIO_Exported_Types */

/** @defgroup ESIO_Exporte                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       <input type="button"                                                                                                                  >d_FunctionPrototypes
  * @{
  */
    void ESIO_SWReset(TSB_ESIO_TypeDef * ESIOx);
    void ESIO_Enable(TSB_ESIO_TypeDef * ESIOx);
    void ESIO_Disable(TSB_ESIO_TypeDef * ESIOx);
    void ESIO_SetTxRxCtrl(TSB_ESIO_TypeDef * ESIOx, FunctionalState NewState);
    FunctionalState ESIO_GetTxRxCtrl(TSB_ESIO_TypeDef * ESIOx);
    void ESIO_SelectMode(TSB_ESIO_TypeDef * ESIOx, ESIO_Mode Mode);
    void ESIO_SelectTransferMode(TSB_ESIO_TypeDef * ESIOx, ESIO_TransferMode TrMode);
    void ESIO_SetCS(TSB_ESIO_TypeDef * ESIOx, ESIO_CSx CSx);
    void ESIO_SetTransferNum(TSB_ESIO_TypeDef * ESIOx, uint8_t Num);
    void ESIO_SetTxPinInIdle(TSB_ESIO_TypeDef * ESIOx, ESIO_TxPinInIdle PinMode);
    void ESIO_SetFIFOLevelINT(TSB_ESIO_TypeDef * ESIOx, uint8_t TxRx, uint8_t Level);
    void ESIO_SetDMA(TSB_ESIO_TypeDef * ESIOx, uint8_t TxRx, FunctionalState NewState);
    void ESIO_SetINT(TSB_ESIO_TypeDef * ESIOx, uint32_t IntSrc, FunctionalState NewState);
    void ESIO_InitFIFO(TSB_ESIO_TypeDef * ESIOx, uint8_t TxRx);
    void ESIO_SetBaudRate(TSB_ESIO_TypeDef * ESIOx, ESIO_BaudClock Clk, uint8_t Divider);
    void ESIO_Init(TSB_ESIO_TypeDef * ESIOx, ESIO_InitTypeDef * Init);
    void ESIO_SetTxData(TSB_ESIO_TypeDef * ESIOx, uint32_t Dat);
    uint32_t ESIO_GetRxData(TSB_ESIO_TypeDef * ESIOx);
    FunctionalState ESIO_IsRegisterModifiable(TSB_ESIO_TypeDef * ESIOx);
    ESIO_StatusFlag ESIO_GetStatus(TSB_ESIO_TypeDef * ESIOx);
    ESIO_ParityErrNum ESIO_GetParityErrorFlag(TSB_ESIO_TypeDef * ESIOx);
    void ESIO_ClrAllParityErrFlag(TSB_ESIO_TypeDef * ESIOx);
    uint32_t ESIO_GetHorizontalParityError(TSB_ESIO_TypeDef * ESIOx, ESIO_LINEx Line);
    void ESIO_ClrHorizontalParityError(TSB_ESIO_TypeDef * ESIOx, ESIO_LINEx Line);
    ESIO_VerticalParityErrFrame ESIO_GetVerticalParityErrFrame(TSB_ESIO_TypeDef * ESIOx);
    void ESIO_ClrAllVerticalParityErrNum(TSB_ESIO_TypeDef * ESIOx);



/** @} */
/* End of group ESIO_Exported_FunctionPrototypes */

/** @} */
/* End of group ESIO */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_ESIO_H) */

#ifdef __cplusplus
}
#endif
#endif                          /* __TMPM440_ESIO_H */
