/**
 *******************************************************************************
 * @file    tmpm440_cg.c
 * @brief   This file provides API functions for CG driver 
 * @version V2.0.2.3
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_cg.h"

#if defined(__TMPM440_CG_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup CG
  * @brief CG driver modules
  * @{
  */

/** @defgroup CG_Private_Defines
  * @{
  */
#ifndef  __BIG_ENDIAN
#ifdef __IAR_SYSTEMS_ICC__
#if 1 != __LITTLE_ENDIAN__
#define     __BIG_ENDIAN
#endif
#endif
#endif

#define CG_FC_GEAR_MASK             ((uint32_t)0xFFFFFFF8)
#define CG_PRCK_MASK                ((uint32_t)0xFFFFF8FF)
#define CG_SCOUT_MASK               ((uint32_t)0x00030000)
#define CG_WUP_TIME_MASK            ((uint32_t)0x0007070F)

#define CG_STBY_MODE_MASK           ((uint32_t)0xFFFFFFF8)
#define CG_RESET_FLAG_MASK          ((uint32_t)0x5F)
#define CG_FPSEL_MASK               ((uint32_t)0xFFFFEFFF)

#define FC_GEAR_1_1                 ((uint32_t)0x00000000)
#define FC_GEAR_1_2                 ((uint32_t)0x00000004)
#define FC_GEAR_1_4                 ((uint32_t)0x00000005)
#define FC_GEAR_1_8                 ((uint32_t)0x00000006)
#define FC_GEAR_1_16                ((uint32_t)0x00000007)

#define WARM_UP_SEL_OSC_EXT         ((uint32_t)0x00080000)
#define WARM_UP_SEL_OSC_INT         ((uint32_t)0x00000000)

#define FTMRD_GEAR_1_1              ((uint32_t)0x00000000)
#define FTMRD_GEAR_1_2              ((uint32_t)0x00000010)
#define FTMRD_GEAR_1_4              ((uint32_t)0x00000020)

#define CG_PROTECT_SET              ((uint32_t)0x000000C1)
#define CG_PROTECT_CLEAR            ((uint32_t)0x0000003E)

typedef union {
    uint32_t byte4;
    uint8_t byte[4];
} CG_Byte4;

static CG_DivideLevel numToDivideLevel_table[CG_DIVIDE_MAX] = {
    CG_DIVIDE_1,
    CG_DIVIDE_2,
    CG_DIVIDE_4,
    CG_DIVIDE_8,
    CG_DIVIDE_16,
    CG_DIVIDE_32,
    CG_DIVIDE_64,
    CG_DIVIDE_128,
    CG_DIVIDE_256,
    CG_DIVIDE_512,
    CG_DIVIDE_UNKNOWN,
};

static CG_PhiT0Src numPhiT0Src_table[CG_PHIT0_SRC_MAX] = {
    CG_PHIT0_SRC_FGEAR,
    CG_PHIT0_SRC_FC,
};

static CG_STBYMode numToSTBYMode_table[CG_STBY_MODE_MAX] = {
    CG_STBY_MODE_UNKNOWN,
    CG_STBY_MODE_STOP1,
    CG_STBY_MODE_UNKNOWN,
    CG_STBY_MODE_IDLE,
    CG_STBY_MODE_UNKNOWN,
    CG_STBY_MODE_STOP2,
    CG_STBY_MODE_UNKNOWN,
    CG_STBY_MODE_UNKNOWN,
};

static volatile uint32_t *const CG_IMCGx[8U] = {
    &TSB_CG->IMCGA, &TSB_CG->IMCGB, &TSB_CG->IMCGC,
    &TSB_CG->IMCGD, &TSB_CG->IMCGE, &TSB_CG->IMCGF,
    &TSB_CG->IMCGG, &TSB_CG->IMCGH
};

static CG_INTActiveState CG_Int_Active_State_Table[4U] = {
    CG_INT_ACTIVE_STATE_INVALID,
    CG_INT_ACTIVE_STATE_RISING,
    CG_INT_ACTIVE_STATE_FALLING,
    CG_INT_ACTIVE_STATE_BOTH_EDGES
};

/** @} */
/* End of group CG_Private_Defines */

/** @defgroup CG_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group CG_Private_FunctionPrototypes */

/** @defgroup CG_Private_Functions
  * @{
  */

/** @} */
/* End of group CG_Private_Functions */

/** @defgroup CG_Exported_Functions
  * @{
  */

/**
  * @brief  Set dividing level between clock fgear and fc.
  * @param  DivideFgearFromFc: Dividing level between fgear and fc.
  *   This parameter can be one of the following values:
  *   CG_DIVIDE_1, CG_DIVIDE_2, CG_DIVIDE_4, CG_DIVIDE_8, CG_DIVIDE_16
  * @retval None
  */
void CG_SetFgearLevel(CG_DivideLevel DivideFgearFromFc)
{
    uint32_t gear = FC_GEAR_1_1;
    uint32_t regval = TSB_CG->SYSCR;

    /* Check the parameters */
    assert_param(IS_CG_GEAR_DIVIDE_LEVEL(DivideFgearFromFc));

    /* Set the value of fgear */
    switch (DivideFgearFromFc) {
    case CG_DIVIDE_1:
        gear = FC_GEAR_1_1;
        break;
    case CG_DIVIDE_2:
        gear = FC_GEAR_1_2;
        break;
    case CG_DIVIDE_4:
        gear = FC_GEAR_1_4;
        break;
    case CG_DIVIDE_8:
        gear = FC_GEAR_1_8;
        break;
    case CG_DIVIDE_16:
        gear = FC_GEAR_1_16;
        break;
    default:
        /* Do nothing */
        break;
    }

    regval &= CG_FC_GEAR_MASK;
    regval |= gear;
    TSB_CG->SYSCR = regval;
}

/**
  * @brief  Get dividing level between clock fgear and fc.
  * @param  None
  * @retval The dividing level between clock fgear and fc
  *   The value returned can be one of the following values:
  *   CG_DIVIDE_1, CG_DIVIDE_2, CG_DIVIDE_4,
  *   CG_DIVIDE_8, CG_DIVIDE_16 or CG_DIVIDE_UNKNOWN
  */
CG_DivideLevel CG_GetFgearLevel(void)
{
    CG_DivideLevel DivideFgearFromFc = CG_DIVIDE_UNKNOWN;
    uint32_t syscr = TSB_CG->SYSCR;
    syscr &= (~CG_FC_GEAR_MASK);

    switch (syscr) {
    case FC_GEAR_1_1:
        DivideFgearFromFc = CG_DIVIDE_1;
        break;
    case FC_GEAR_1_2:
        DivideFgearFromFc = CG_DIVIDE_2;
        break;
    case FC_GEAR_1_4:
        DivideFgearFromFc = CG_DIVIDE_4;
        break;
    case FC_GEAR_1_8:
        DivideFgearFromFc = CG_DIVIDE_8;
        break;
    case FC_GEAR_1_16:
        DivideFgearFromFc = CG_DIVIDE_16;
        break;
    default:
        /* Do nothing */
        break;
    }

    return DivideFgearFromFc;
}

/**
  * @brief  Set fperiph for PhiT0
  * @param  PhiT0Src: Set PhiT0 Source
  *   This parameter can be one of the following values:
  *   CG_PHIT0_SRC_FGEAR or CG_PHIT0_SRC_FC
  * @retval None
  */
void CG_SetPhiT0Src(CG_PhiT0Src PhiT0Src)
{
    uint32_t regval = 0U;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_CG_PHIT0_SRC(PhiT0Src));

    regval = TSB_CG->SYSCR;
    regval &= CG_FPSEL_MASK;
    tmp = (uint32_t) PhiT0Src;
    regval |= (tmp << 12U);

    TSB_CG->SYSCR = regval;
}

/**
  * @brief  Get PhiT0 Source
  * @param  None
  * @retval The source of PhiT0
  *   The value returned can be one of the following values:
  *   CG_PHIT0_SRC_FGEAR or CG_PHIT0_SRC_FC
  */
CG_PhiT0Src CG_GetPhiT0Src(void)
{
    uint32_t regval = 0U;
    CG_PhiT0Src source = CG_PHIT0_SRC_FGEAR;

    regval = TSB_CG->SYSCR;
    regval &= (~CG_FPSEL_MASK);
    regval = (regval >> 12U);
    source = numPhiT0Src_table[regval];

    return source;
}

/**
  * @brief  Set dividing level between clock PhiT0 and fc.
  * @param  DividePhiT0FromFc: Dividing level between PhiT0 and fc.
  *   This parameter can be one of the following values:
  *    CG_DIVIDE_1, 
  *    CG_DIVIDE_2, 
  *    CG_DIVIDE_4, 
  *    CG_DIVIDE_8,
  *    CG_DIVIDE_16, 
  *    CG_DIVIDE_32, 
  *    CG_DIVIDE_64, 
  *    CG_DIVIDE_128,
  *    CG_DIVIDE_256, 
  *    CG_DIVIDE_512.
  * @retval  Success or not
  *   The value returned can be one of the following values:
  *   SUCCESS or ERROR
  */
Result CG_SetPhiT0Level(CG_DivideLevel DividePhiT0FromFc)
{
    uint32_t fprclk = 0U;
    CG_DivideLevel fgear = CG_DIVIDE_UNKNOWN;
    CG_PhiT0Src PhiT0_src = CG_PHIT0_SRC_FGEAR;
    Result retval = ERROR;
    uint32_t regval = TSB_CG->SYSCR;

    PhiT0_src = CG_GetPhiT0Src();
    if (PhiT0_src == CG_PHIT0_SRC_FC) { /* phiT0 src == fc , max divider: 32 */
        /* Check the parameters */
        assert_param(IS_CG_DIVIDE_FC_LEVEL(DividePhiT0FromFc));

        fprclk = (uint32_t) DividePhiT0FromFc;
        regval &= CG_PRCK_MASK;
        fprclk <<= 8U;
        regval |= fprclk;
        TSB_CG->SYSCR = regval;
        retval = SUCCESS;
    } else {                    /* phiT0 src == fgear, max divider: 512 */
        /* Check the parameters */
        assert_param(IS_CG_DIVIDE_FGEAR_LEVEL(DividePhiT0FromFc));

        fgear = CG_GetFgearLevel();
        if (fgear != CG_DIVIDE_UNKNOWN) {
            if (DividePhiT0FromFc >= fgear) {
                fprclk = (uint32_t) DividePhiT0FromFc - (uint32_t) fgear;
                if (fprclk < CG_DIVIDE_64) {
                    regval &= CG_PRCK_MASK;
                    fprclk <<= 8U;
                    regval |= fprclk;
                    TSB_CG->SYSCR = regval;
                    retval = SUCCESS;
                } else {
                    /* Do nothing */
                }
            } else {
                /* Do nothing */
            }
        } else {
            /* Do nothing */
        }
    }

    return retval;
}

/**
  * @brief  Get dividing level between clock phiT0 and fc.
  * @param  None
  * @retval The divide level between clock phiT0 and fc
  *   The value returned can be one of the following values:
  *    CG_DIVIDE_1, 
  *    CG_DIVIDE_2, 
  *    CG_DIVIDE_4, 
  *    CG_DIVIDE_8,
  *    CG_DIVIDE_16, 
  *    CG_DIVIDE_32, 
  *    CG_DIVIDE_64, 
  *    CG_DIVIDE_128,
  *    CG_DIVIDE_256, 
  *    CG_DIVIDE_512,
  *    CG_DIVIDE_UNKNOWN.
  */
CG_DivideLevel CG_GetPhiT0Level(void)
{
    uint32_t fprclk = 0U;
    CG_DivideLevel fgear = CG_DIVIDE_UNKNOWN;
    CG_DivideLevel phiT0Level = CG_DIVIDE_UNKNOWN;
    CG_PhiT0Src PhiT0_src = CG_PHIT0_SRC_FGEAR;

    fprclk = TSB_CG->SYSCR & (~CG_PRCK_MASK);
    fprclk = (uint32_t) (fprclk >> 8U);
    PhiT0_src = CG_GetPhiT0Src();

    if (PhiT0_src == CG_PHIT0_SRC_FC) {
        phiT0Level = numToDivideLevel_table[fprclk];
    } else {
        fgear = CG_GetFgearLevel();
        if (fgear != CG_DIVIDE_UNKNOWN) {
            fprclk = fprclk + fgear;
            if (fprclk < CG_DIVIDE_UNKNOWN) {
                phiT0Level = numToDivideLevel_table[fprclk];
            } else {
                /* Do nothing */
            }
        } else {
            /* Do nothing */
        }
    }

    return phiT0Level;
}

/**
  * @brief Set the source for the clock output from SCOUT pin
  * @param Source: Select source of SCOUT
  *   This parameter can be one of the following values:
  *     CG_SCOUT_FC_DIVIDE_4,
  *     CG_SCOUT_FC_DIVIDE_8,
  *     CG_SCOUT_FOSC,
  *     CG_SCOUT_LOW    
  * @retval None
  */
void CG_SetSCOUTSrc(CG_SCOUTSrc Source)
{
    /* Clear the original SCOUT SRC value */
    uint32_t tmp = TSB_CG->SYSCR & ((uint32_t) (~CG_SCOUT_MASK));

    /* Check the parameters */
    assert_param(IS_CG_SCOUT_SRC(Source));

    tmp |= (uint32_t) Source << 16U;
    TSB_CG->SYSCR = tmp;
}

/**
  * @brief  Get the source of SCOUT output.
  * @param  None
  * @retval Source of SCOUT output
  *   The value returned can be one of the following values:
  *     CG_SCOUT_FC_DIVIDE_4,
  *     CG_SCOUT_FC_DIVIDE_8,
  *     CG_SCOUT_FOSC,
  *     CG_SCOUT_LOW 
  */
CG_SCOUTSrc CG_GetSCOUTSrc(void)
{
    return (CG_SCOUTSrc) ((TSB_CG->SYSCR & CG_SCOUT_MASK) >> 16U);
}

/**
  * @brief  Set the warm up time 
  * @param  Source: Select source of warm-up counter
  *   This parameter can be one of the following values:
  *    CG_WARM_UP_SRC_OSC_EXT or CG_WARM_UP_SRC_OSC_INT
  *    ( Warm-up function is not necessary when using stable external clock.)
  * @param  Time: Set count time for warm-up timer
  * @retval None
  */
void CG_SetWarmUpTime(CG_WarmUpSrc Source, uint16_t Time)
{
    uint32_t tmp = 0U;
    uint32_t regval = TSB_CG->OSCCR;

    /* Check the parameters */
    assert_param(IS_CG_WARM_UP_SRC(Source));

    regval &= CG_WUP_TIME_MASK;

    switch (Source) {
    case CG_WARM_UP_SRC_OSC_EXT:
        /*Round lower 4 bit off, set the value to CGOSCCR<WUODR[11:0]> */
        tmp = (uint32_t) Time >> 4U;
        tmp <<= 20U;
        regval |= tmp;
        regval |= WARM_UP_SEL_OSC_EXT;
        break;
    case CG_WARM_UP_SRC_OSC_INT:
        /*Round lower 4 bit off, set the value to CGOSCCR<WUODR[11:0]> */
        tmp = (uint32_t) Time >> 4U;
        tmp <<= 20U;
        regval |= tmp;
        regval |= WARM_UP_SEL_OSC_INT;
        break;

    default:
        /* Do nothing */
        break;
    }

    TSB_CG->OSCCR = regval;
}

/**
  * @brief  Start operation of warm up timer for oscillator.
  * @param  None
  * @retval None
  */
void CG_StartWarmUp(void)
{
    TSB_CG_OSCCR_WUEON = 1U;
}

/**
  * @brief Check whether warm up is completed or not.
  * @param  None
  * @retval The state of warm-up
  *   The value returned can be one of the following values:
  *   DONE or BUSY
  */
WorkState CG_GetWarmUpState(void)
{
    WorkState state = BUSY;

    if (TSB_CG_OSCCR_WUEF == 0U) {
        state = DONE;
    } else {
        /* Do nothing */
    }

    return state;
}

/**
  * @brief  Set multiplying value for PLL0 (for fsys)
  * @param  NewValue: PLL multiplying value
  *   This parameter can be one of the following values:
  *     CG_FPLL0_IN8_OUT40,
  *     CG_FPLL0_IN8_OUT48,
  *     CG_FPLL0_IN8_OUT64,
  *     CG_FPLL0_IN8_OUT80,
  *     CG_FPLL0_IN10_OUT50,
  *     CG_FPLL0_IN10_OUT60,
  *     CG_FPLL0_IN10_OUT80,
  *     CG_FPLL0_IN10_OUT100 
  * @retval Success or not
  *   The value returned can be one of the following values:
  *     SUCCESS or ERROR
  */
Result CG_SetFPLLValue(uint32_t NewValue)
{
    Result retval = SUCCESS;

    /* read PLLSEL and clear _PLL0SET(bit15:1) */
    uint32_t tmp = TSB_CG->PLLSEL & 0xFFFF0001U;

    /* Check the parameters */
    assert_param(IS_CG_FPLL_VALUE(NewValue));

    /* Don't use the PLL when internal high-speed oscillator (IHOSC) is used as system clock */
    /* When PLL is on, don't change the PLL setting value */
    if (CG_FOSC_OSC_INT == CG_GetFoscSrc()) {
        retval = ERROR;
    } else if (ENABLE == CG_GetPLLState()) {
        retval = ERROR;
    } else {
        tmp |= NewValue;
        TSB_CG->PLLSEL = tmp;
    }

    return retval;
}


/**
  * @brief  Enable PLL0 for fsys or disable it.
  * @param  NewState: New state of PLL0
  *   This parameter can be one of the following values:
  *     DISABLE or ENABLE
  * @retval Success or not
  *   The value returned can be one of the following values:
  *     SUCCESS or ERROR
  */
Result CG_SetPLL(FunctionalState NewState)
{
    Result retval = SUCCESS;
    uint32_t tmp = 10000U;
    CG_FcSrc fcsrc = CG_FC_SRC_FPLL;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG_OSCCR_PLL0ON = 1U;
        while (tmp) {           /* wait at least 100uS for PLL stable */
            tmp--;
        }
    } else {
        fcsrc = CG_GetFcSrc();
        if (fcsrc == CG_FC_SRC_FOSC) {  /* PLL is not used. So it can be shut off */
            /* Set register to disable PLL */
            TSB_CG_OSCCR_PLL0ON = 0U;
        } else {                /* fcsrc == CG_FC_SRC_FPLL. */
            /* PLL is in use, so disabling it should be invalid */
            retval = ERROR;
        }
    }

    return retval;
}

/**
  * @brief  Get the status of PLL0 operation
  * @param  None
  * @retval Run or stop
  *   The value returned can be one of the following values:
  *     ENABLE or DISABLE
  */
FunctionalState CG_GetPLLState(void)
{
    return (FunctionalState) (TSB_CG_OSCCR_PLL0ON);
}


/**
  * @brief  Set multiplying value for PLL1 (for ADC)
  * @param  NewValue: PLL multiplying value
  *   This parameter can be one of the following values:
  *     CG_FPLL1_IN8_OUT64,
  *     CG_FPLL1_IN8_OUT80,
  *     CG_FPLL1_IN10_OUT80,
  *     CG_FPLL1_IN10_OUT100 
  * @retval Success or not
  *   The value returned can be one of the following values:
  *     SUCCESS or ERROR
  */
Result CG_SetFPLLForADCValue(uint32_t NewValue)
{
    Result retval = SUCCESS;

    /* read PLLSEL and clear _PLL1SET(bit31:17) */
    uint32_t tmp = TSB_CG->PLLSEL & 0x0001FFFFU;

    /* Check the parameters */
    assert_param(IS_CG_FPLL1_VALUE(NewValue));

    /* When PLL is on, don't change the PLL setting value */
    if (TSB_CG_OSCCR_PLL1ON) {
        retval = ERROR;
    } else {
        tmp |= NewValue;
        TSB_CG->PLLSEL = tmp;
    }

    return retval;
}


/**
  * @brief  Set the clock source of ADC
  * @param  FadcSrc: Select clock source for ADC
  *   This parameter can be one of the following values:
  *    CG_FADC_SRC_FC, 
  *    CG_FADC_SRC_FPLL,
  * @retval None
  */
void CG_SetFadcSrc(CG_FadcSrc FadcSrc)
{
    /* Check the parameters */
    assert_param(IS_CG_FADC_SRC(FadcSrc));

    /* Set the value of fgear */
    if (FadcSrc == CG_FADC_SRC_FPLL) {
        TSB_CG_PLLSEL_PLL1SEL = 1U;
    } else {
        TSB_CG_PLLSEL_PLL1SEL = 0U;
    }
}


/**
  * @brief  Enable PLL1 for ADC or disable it.
  * @param  NewState: New state of PLL1
  *   This parameter can be one of the following values:
  *     DISABLE or ENABLE
  * @retval None
  */
void CG_SetPLL1ForADC(FunctionalState NewState)
{
    uint32_t tmp = 10000U;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG_OSCCR_PLL1ON = 1U;
        while (tmp) {           /* wait at least 100uS for PLL stable */
            tmp--;
        }
    } else {
        TSB_CG_OSCCR_PLL1ON = 0U;
    }
}

/**
  * @brief  Enable or disable high-speed oscillator (fosc)
  * @param  Source: Select clock source for fosc
  *   This parameter can be one of the following values:
  *    CG_FOSC_OSC_EXT or CG_FOSC_OSC_INT
  * @param  NewState: oscillator is enabled or disabled
  *   This parameter can be one of the following values:
  *    DISABLE or ENABLE
  * @retval Success or failure
  *   The value returned can be one of the following values:
  *    SUCCESS or ERROR
  */
Result CG_SetFosc(CG_FoscSrc Source, FunctionalState NewState)
{
    CG_FoscSrc fosc_src = CG_FOSC_OSC_INT;
    Result retval = ERROR;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_CG_FOSC_STATE(Source));

    switch (Source) {
    case CG_FOSC_OSC_EXT:
        if (NewState == ENABLE) {
            /* Enable external high-speed oscillator */
            TSB_CG_OSCCR_XEN1 = 1U;
            /*Selects external high-speed oscillator */
            TSB_CG_OSCCR_EHOSCSEL = 1U;
            retval = SUCCESS;
        } else {
            fosc_src = CG_GetFoscSrc();
            if (fosc_src == CG_FOSC_OSC_EXT) {
                /* external oscillator is in  use, so it can't be disabled */
                retval = ERROR;
            } else {
                /* Disable external high-speed oscillator */
                TSB_CG_OSCCR_XEN1 = 0U;
                retval = SUCCESS;
            }
        }

        break;
    case CG_FOSC_OSC_INT:
        if (NewState == ENABLE) {
            /* Enable internal high-speed oscillator */
            TSB_CG_OSCCR_XEN2 = 1U;
            retval = SUCCESS;
        } else {
            fosc_src = CG_GetFoscSrc();
            if (fosc_src == CG_FOSC_OSC_INT) {
                /* internal high-speed oscillator is in  use, so it can't be disabled */
                retval = ERROR;
            } else {
                /* Disable internal high-speed oscillator */
                TSB_CG_OSCCR_XEN2 = 0U;
                retval = SUCCESS;
            }
        }
        break;
    default:
        /* Do nothing */
        break;
    }

    return retval;
}

/**
  * @brief  Set the source of high-speed oscillator (fosc)
  * @param  Source: Select clock source for fosc
  *   This parameter can be one of the following values:
  *    CG_FOSC_OSC_EXT , CG_FOSC_CLKIN_EXT or CG_FOSC_OSC_INT
  * @retval None
  */
void CG_SetFoscSrc(CG_FoscSrc Source)
{
    /* Check the parameters */
    assert_param(IS_CG_FOSC_SRC(Source));

    if (Source == CG_FOSC_OSC_INT) {
        /*Selects internal high-speed oscillator */
        TSB_CG_OSCCR_OSCSEL = 0U;
    } else {
        if (Source == CG_FOSC_OSC_EXT) {
            /*Selects external high-speed oscillator */
            TSB_CG_OSCCR_EHOSCSEL = 1U;
            /*Selects external high-speed oscillator */
            TSB_CG_OSCCR_OSCSEL = 1U;
        } else {
            /*Selects an external clock input */
            TSB_CG_OSCCR_EHOSCSEL = 0U;
            /*Selects external high-speed oscillator */
            TSB_CG_OSCCR_OSCSEL = 1U;
        }
    }
}

/**
  * @brief  Get the source of high-speed oscillator (fosc)
  * @param  None
  * @retval Source of fosc
  *   The value returned can be one of the following values:
  *    CG_FOSC_OSC_EXT , CG_FOSC_CLKIN_EXT or CG_FOSC_OSC_INT
  */
CG_FoscSrc CG_GetFoscSrc(void)
{
    CG_FoscSrc fosc_src = CG_FOSC_CLKIN_EXT;

    if (TSB_CG_OSCCR_OSCSEL == 0U) {
        fosc_src = CG_FOSC_OSC_INT;
    } else {
        if (TSB_CG_OSCCR_EHOSCSEL == 1U) {
            fosc_src = CG_FOSC_OSC_EXT;
        } else {
            /* Do nothing */
        }
    }

    return fosc_src;
}

/**
  * @brief  Get the state of high-speed oscillator (fosc)
  * @param  Source: Select fosc source
  *   This parameter can be one of the following values:
  *    CG_FOSC_OSC_EXT or CG_FOSC_OSC_INT
  * @retval State of fosc
  *   The value returned can be one of the following values:
  *    ENABLE or DISABLE
  */
FunctionalState CG_GetFoscState(CG_FoscSrc Source)
{
    FunctionalState state = DISABLE;

    /* Check the parameters */
    assert_param(IS_CG_FOSC_STATE(Source));

    if (Source == CG_FOSC_OSC_EXT) {
        if (TSB_CG_OSCCR_XEN1 == 1U) {
            state = ENABLE;
        } else {
            /* Do nothing */
        }
    } else if (TSB_CG_OSCCR_XEN2 == 1U) {
        state = ENABLE;
    } else {
        /* Do nothing */
    }

    return state;
}


/**
  * @brief  enable or disable low-speed oscillation (fs)
  *         only used for RTC, KWUP, KSCAN, Warm-up.
  * @param NewState: oscillation is enabled or disabled
  *   This parameter can be one of the following values:
  *    DISABLE or ENABLE.
  * @retval success or failure
  *   The value returned can be one of the following values:
  *    SUCCESS or ERROR.
  */
Result CG_SetFs(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    TSB_CG_OSCCR_XTEN = (uint32_t) NewState;

    return SUCCESS;
}

/**
  * @brief  get the state of low-speed oscillation (fs)
  * @param None
  * @retval state of fs
  *   The value returned can be one of the following values:
  *    DISABLE or ENABLE.
  */
FunctionalState CG_GetFsState(void)
{
    return (FunctionalState) TSB_CG_OSCCR_XTEN;
}



/**
  * @brief  Set to the specified low-power mode
  * @param  Low power mode
  *   The value can be one of the following values:
  *    CG_STBY_MODE_STOP1, CG_STBY_MODE_IDLE, CG_STBY_MODE_STOP2
  * @retval None
  */
void CG_SetSTBYMode(CG_STBYMode Mode)
{
    uint32_t regval = TSB_CG->STBYCR;

    /* Check the parameter */
    assert_param(IS_CG_STBY_MODE(Mode));

    regval &= CG_STBY_MODE_MASK;
    regval |= (uint32_t) Mode;

    TSB_CG->STBYCR = regval;
}

/**
  * @brief  Get the low-power consumption mode
  * @param  None
  * @retval Low power mode
  *   The value returned can be one of the following values:
  *    CG_STBY_MODE_STOP1, CG_STBY_MODE_IDLE, 
  *    CG_STBY_MODE_STOP2, CG_STBY_MODE_UNKNOWN
  */
CG_STBYMode CG_GetSTBYMode(void)
{
    CG_STBYMode stby_mode = CG_STBY_MODE_UNKNOWN;
    uint8_t regval = (uint8_t) (TSB_CG->STBYCR & (~CG_STBY_MODE_MASK));

    stby_mode = numToSTBYMode_table[regval];

    return stby_mode;
}

/**
  * @brief  Set pin status in stop1 mode
  * @param  NewState: enable or disable
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE
  * @retval None
  */
void CG_SetPinStateInStop1Mode(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG_STBYCR_DRVE = 1U;
    } else {
        TSB_CG_STBYCR_DRVE = 0U;
    }
}

/**
  * @brief  Get pin status in stop1 mode
  * @param  None
  * @retval State of pin in stop mode
  *   The value returned can be one of the following values:
  *    DISABLE or ENABLE
  */
FunctionalState CG_GetPinStateInStop1Mode(void)
{
    FunctionalState state = DISABLE;

    if (TSB_CG_STBYCR_DRVE == 1U) {
        /* Pin status in stop mode is active */
        state = ENABLE;
    } else {
        /* Do nothing */
    }
    return state;
}

/**
  * @brief  Enables or disables to keep IO control signal in stop2 mode
  * @param  NewState: enable or disable
  *   This parameter can be one of the following values:
  *    ENABLE or DISABLE
  * @retval None
  */
void CG_SetPortKeepInStop2Mode(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG_STBYCR_PTKEEP = 1U;
    } else {
        TSB_CG_STBYCR_PTKEEP = 0U;
    }
}

/**
  * @brief  Get the status of  IO control signal in stop2 mode
  * @param  None
  * @retval State of IO control signal in stop2 mode
  *   The value returned can be one of the following values:
  *    DISABLE or ENABLE
  */
FunctionalState CG_GetPortKeepInStop2Mode(void)
{
    FunctionalState state = DISABLE;

    if (TSB_CG_STBYCR_PTKEEP == 1U) {
        /* Pin status in stop mode is active */
        state = ENABLE;
    } else {
        /* Do nothing */
    }
    return state;
}

/**
  * @brief  Set the source of fc
  * @param  Source:  The source of fc
  *   This parameter can be one of the following values:
  *    CG_FC_SRC_FOSC or CG_FC_SRC_FPLL
  * @retval success or failure
  *   The value returned can be one of the following values:
  *    SUCCESS or ERROR
  */
Result CG_SetFcSrc(CG_FcSrc Source)
{
    Result retval = ERROR;
    CG_FoscSrc fosc_src = CG_FOSC_OSC_EXT;

    /* Check the parameters */
    assert_param(IS_CG_FC_SRC(Source));

    if (Source == CG_FC_SRC_FOSC) {
        fosc_src = CG_GetFoscSrc();
        switch (fosc_src) {
        case CG_FOSC_OSC_EXT:
            if (ENABLE == CG_GetFoscState(CG_FOSC_OSC_EXT)) {
                TSB_CG_PLLSEL_PLL0SEL = 0U;
                retval = SUCCESS;
            } else {
                /* Do nothing */
            }
            break;
        case CG_FOSC_OSC_INT:
            if (ENABLE == CG_GetFoscState(CG_FOSC_OSC_INT)) {
                TSB_CG_PLLSEL_PLL0SEL = 0U;
                retval = SUCCESS;
            } else {
                /* Do nothing */
            }
            break;
        case CG_FOSC_CLKIN_EXT:
            TSB_CG_PLLSEL_PLL0SEL = 0U;
            retval = SUCCESS;
            break;
        default:
            /* Do nothing */
            break;
        }

    } else {
        if (ENABLE == CG_GetPLLState()) {       /* if PLL0 is On and stable, can select it */
            TSB_CG_PLLSEL_PLL0SEL = 1U;
            retval = SUCCESS;
        } else {
            /* Do nothing */
        }
    }

    return retval;
}

/**
  * @brief  Get the clock source of fc
  * @param  None
  * @retval The clock source of fc
  *   The value returned can be one of the following values:
  *    CG_FC_SRC_FOSC or CG_FC_SRC_FPLL
  */
CG_FcSrc CG_GetFcSrc(void)
{
    return (CG_FcSrc) (TSB_CG_PLLSEL_PLL0SEL);
}

/**
  * @brief  Set the clock source of high-resolution timer TMRD 
  * @param  TmrdUnit: Select unit for TIMER-D
  *   This parameter can be one of the following values:
  *    CG_TMRD_UNIT_A 
  * @param  FtmrdSrc: Select clock source for ftmrd
  *   This parameter can be one of the following values:
  *    CG_FTMRD_SRC_FC, 
  *    CG_FTMRD_SRC_HALF_FC,
  *    CG_FTMRD_SRC_QUARTER_FC 
  * @retval None
  */
void CG_SetFtmrdSrc(CG_TmrdUnit TmrdUnit, CG_FtmrdSrc FtmrdSrc)
{
    uint32_t tmp = TSB_CG->PWMGEAR & (uint32_t) (~(0x03U << 4U));

    /* Check the parameters */
    /* assert_param(IS_CG_TMRD_UNIT(TmrdUnit)); */
    assert_param(IS_CG_FTMRD_SRC(FtmrdSrc));

    /* Set the value of fgear */
    TSB_CG_PWMGEAR_TMRDCLKEN = 0U;      /* stop TMRD clock before setting */
    tmp |= ((uint32_t) (FtmrdSrc)) << 4U;
    TSB_CG->PWMGEAR = tmp;      /* later call CG_SetTMRDClk() to enable TMRD clock */
}

/**
  * @brief  Get the clock source of high-resolution timer TMRD 
  * @param  TmrdUnit: Select unit for TIMER-D
  *   This parameter can be one of the following values:
  *    CG_TMRD_UNIT_A .
  * @retval Source of ftmrd
  *   The value returned can be one of the following values:
  *    CG_FTMRD_SRC_FC, 
  *    CG_FTMRD_SRC_HALF_FC,
  *    CG_FTMRD_SRC_QUARTER_FC,
  *    CG_FTMRD_SRC_UNKNOWN
  */
CG_FtmrdSrc CG_GetFtmrdSrc(CG_TmrdUnit TmrdUnit)
{
    uint32_t tmp = (TSB_CG->PWMGEAR & (uint32_t) (0x03U << 4U)) >> 4U;

    /* Check the parameters */
    /* assert_param(IS_CG_TMRD_UNIT(TmrdUnit)); */

    return (CG_FtmrdSrc) tmp;
}

/**
  * @brief  Enable or disable to provide TMRD clock
  * @param  TmrdUnit: Select unit for TIMER-D
  *   This parameter can be one of the following values:
  *    CG_TMRD_UNIT_A 
  * @param  NewState: New state of TMRD clock
  *   This parameter can be one of the following values:
  *    DISABLE or ENABLE
  * @retval None
  */
void CG_SetTMRDClk(CG_TmrdUnit TmrdUnit, FunctionalState NewState)
{
    /* Check the parameters */
    /* assert_param(IS_CG_TMRD_UNIT(TmrdUnit)); */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    TSB_CG_PWMGEAR_TMRDCLKEN = (uint32_t) NewState;

}

/**
  * @brief  Get the state of TMRD clock
  * @param  TmrdUnit: Select unit for TIMER-D
  *   This parameter can be one of the following values:
  *    CG_TMRD_UNIT_A.
  * @retval state of TMRD clock
  *   The value returned can be one of the following values:
  *    ENABLE or DISABLE
  */
FunctionalState CG_GetTMRDClkState(CG_TmrdUnit TmrdUnit)
{
    /* Check the parameters */
    /* assert_param(IS_CG_TMRD_UNIT(TmrdUnit)); */

    return (FunctionalState) TSB_CG_PWMGEAR_TMRDCLKEN;
}

/**
  * @brief  Enable or disable to protect CG registers
  * @param  NewState: New state of PLL
  *   This parameter can be one of the following values:
  *    DISABLE or ENABLE
  * @retval None
  */
void CG_SetProtectCtrl(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_CG->PROTECT = CG_PROTECT_SET;
    } else {
        TSB_CG->PROTECT = CG_PROTECT_CLEAR;
    }
}


/**
  * @brief  Gets the active state of INT source standby clear request
  * @param  INTSource: Select the release INT source
  *   This parameter can be one of the following values:
  *    CG_INT_SRC_0, CG_INT_SRC_1, CG_INT_SRC_2,
  *    CG_INT_SRC_3, CG_INT_SRC_4, CG_INT_SRC_5,
  *    CG_INT_SRC_6, CG_INT_SRC_7, CG_INT_SRC_8,
  *    CG_INT_SRC_9, CG_INT_SRC_A, CG_INT_SRC_B,
  *    CG_INT_SRC_C, CG_INT_SRC_D, CG_INT_SRC_E,
  *    CG_INT_SRC_F, CG_INT_SRC_10, CG_INT_SRC_11,
  *    CG_INT_SRC_12, CG_INT_SRC_13, CG_INT_SRC_14, 
  *    CG_INT_SRC_15,
  *    CG_INT_SRC_INTKWUP0,
  *    CG_INT_SRC_INTKWUP1, 
  *    CG_INT_SRC_INTKSCAN, 
  *    CG_INT_SRC_INTRTC,
  *    CG_INT_SRC_INTPHC00, 
  *    CG_INT_SRC_INTPHC01, 
  *    CG_INT_SRC_INTPHC0EVRY,
  *    CG_INT_SRC_INTPHC10, 
  *    CG_INT_SRC_INTPHC11, 
  *    CG_INT_SRC_INTPHC1EVRY.
  * @retval Active state of the input INT
  *   The value returned can be one of the following values:
  *     CG_INT_ACTIVE_STATE_L, 
  *     CG_INT_ACTIVE_STATE_H, 
  *     CG_INT_ACTIVE_STATE_FALLING, 
  *     CG_INT_ACTIVE_STATE_RISING, 
  *     CG_INT_ACTIVE_STATE_BOTH_EDGES.
  */
CG_INTActiveState CG_GetSTBYReleaseINTState(CG_INTSrc INTSource)
{
    uint32_t i = 0U;
    uint32_t j = 0U;
    CG_Byte4 regval = { 0U };
    CG_INTActiveState state;
    /* Check the parameters */
    assert_param(IS_CG_INT_SRC(INTSource));

    i = ((uint32_t) INTSource) / 4U;

#ifdef  __BIG_ENDIAN
    j = 3U - ((uint32_t) INTSource) % 4U;
#else
    j = ((uint32_t) INTSource) % 4U;
#endif
    regval.byte4 = *CG_IMCGx[i];
    /* get EMCGx[1:0] */
    i = (((uint32_t) regval.byte[j] & 0x70U)) >> 4U;
    if (i == 0x04U) {
        /* get EMSTx[1:0] */
        i = (((uint32_t) regval.byte[j] & 0x0CU)) >> 2U;
        if (i == 0U) {
            state = CG_INT_ACTIVE_STATE_INVALID;
        } else {
            state = CG_Int_Active_State_Table[i];
        }
    } else {
        switch (i) {
        case 0x00U:
            state = CG_INT_ACTIVE_STATE_L;
            break;
        case 0x01U:
            state = CG_INT_ACTIVE_STATE_H;
            break;
        case 0x02U:
            state = CG_INT_ACTIVE_STATE_FALLING;
            break;
        case 0x03U:
            state = CG_INT_ACTIVE_STATE_RISING;
            break;
        default:
            state = CG_INT_ACTIVE_STATE_INVALID;
            break;
        }
    }
    return state;
}


/**
  * @brief  Setup the INT source for releasing low power mode.
  * @param  INTSource: Select the release INT source
  *   This parameter can be one of the following values:
  *    CG_INT_SRC_0, CG_INT_SRC_1, CG_INT_SRC_2,
  *    CG_INT_SRC_3, CG_INT_SRC_4, CG_INT_SRC_5,
  *    CG_INT_SRC_6, CG_INT_SRC_7, CG_INT_SRC_8,
  *    CG_INT_SRC_9, CG_INT_SRC_A, CG_INT_SRC_B,
  *    CG_INT_SRC_C, CG_INT_SRC_D, CG_INT_SRC_E,
  *    CG_INT_SRC_F, CG_INT_SRC_10, CG_INT_SRC_11,
  *    CG_INT_SRC_12, CG_INT_SRC_13, CG_INT_SRC_14, 
  *    CG_INT_SRC_15,
  *    CG_INT_SRC_INTKWUP0,
  *    CG_INT_SRC_INTKWUP1, 
  *    CG_INT_SRC_INTKSCAN, 
  *    CG_INT_SRC_INTRTC,
  *    CG_INT_SRC_INTPHC00, 
  *    CG_INT_SRC_INTPHC01, 
  *    CG_INT_SRC_INTPHC0EVRY,
  *    CG_INT_SRC_INTPHC10, 
  *    CG_INT_SRC_INTPHC11, 
  *    CG_INT_SRC_INTPHC1EVRY.
  * @param  ActiveState: select the active state for release trigger
  *   For CG_INT_SRC_0 to CG_INT_SRC_15, 
  *    One of the following values can be set.
  *     CG_INT_ACTIVE_STATE_L, 
  *     CG_INT_ACTIVE_STATE_H, 
  *     CG_INT_ACTIVE_STATE_FALLING, 
  *     CG_INT_ACTIVE_STATE_RISING, 
  *     CG_INT_ACTIVE_STATE_BOTH_EDGES. 
  *   For CG_INT_SRC_INTKWUP0 to CG_INT_SRC_INTKWUP1, 
  *    One of the following values can be set.
  *     CG_INT_ACTIVE_STATE_H, 
  *   For CG_INT_SRC_INTRTC, 
  *    One of the following values can be set.
  *     CG_INT_ACTIVE_STATE_FALLING,  
  *   For the others in INTSource, 
  *    One of the following values can be set.
  *     CG_INT_ACTIVE_STATE_RISING,   
  * @param  NewState: Enable or disable this release trigger
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE
  * @retval None
  */
void CG_SetSTBYReleaseINTSrc(CG_INTSrc INTSource,
                             CG_INTActiveState ActiveState, FunctionalState NewState)
{
    uint32_t i = 0U;
    uint32_t j = 0U;
    CG_Byte4 regval = { 0U };

    /* Check the parameters */
    assert_param(IS_CG_INT_SRC(INTSource));
    if (INTSource <= CG_INT_SRC_15) {
        assert_param(IS_CG_INT_ACTIVE_STATE_A(ActiveState));
    } else if ((INTSource == CG_INT_SRC_INTKWUP0) || (INTSource == CG_INT_SRC_INTKWUP1)) {
        assert_param(IS_CG_INT_ACTIVE_STATE_B(ActiveState));
    } else if (INTSource == CG_INT_SRC_INTRTC) {
        assert_param(IS_CG_INT_ACTIVE_STATE_C(ActiveState));
    } else {
        assert_param(IS_CG_INT_ACTIVE_STATE_D(ActiveState));
    }
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    i = ((uint32_t) INTSource) / 4U;
#ifdef __BIG_ENDIAN
    j = 3U - ((uint32_t) INTSource) % 4U;
#else
    j = ((uint32_t) INTSource) % 4U;
#endif

    /* set ActiveState and NewState to IMCGA to IMCGG one by one */
    regval.byte4 = *CG_IMCGx[i];
    regval.byte[j] &= 0x8CU;    /* clear bit 6, 5, 4, 1, 0 of correct byte */

    regval.byte[j] |= ActiveState;
    *CG_IMCGx[i] = regval.byte4;

    regval.byte[j] |= NewState;
    *CG_IMCGx[i] = regval.byte4;

}

/**
  * @brief  Clears the input INT request.
  * @param  INTSource: Select the release INT source
  *   This parameter can be one of the following values:
  *    CG_INT_SRC_0, CG_INT_SRC_1, CG_INT_SRC_2,
  *    CG_INT_SRC_3, CG_INT_SRC_4, CG_INT_SRC_5,
  *    CG_INT_SRC_6, CG_INT_SRC_7, CG_INT_SRC_8,
  *    CG_INT_SRC_9, CG_INT_SRC_A, CG_INT_SRC_B,
  *    CG_INT_SRC_C, CG_INT_SRC_D, CG_INT_SRC_E,
  *    CG_INT_SRC_F, CG_INT_SRC_10, CG_INT_SRC_11,
  *    CG_INT_SRC_12, CG_INT_SRC_13, CG_INT_SRC_14, 
  *    CG_INT_SRC_15,
  *    CG_INT_SRC_INTKWUP0,
  *    CG_INT_SRC_INTKWUP1, 
  *    CG_INT_SRC_INTKSCAN, 
  *    CG_INT_SRC_INTRTC,
  *    CG_INT_SRC_INTPHC00, 
  *    CG_INT_SRC_INTPHC01, 
  *    CG_INT_SRC_INTPHC0EVRY,
  *    CG_INT_SRC_INTPHC10, 
  *    CG_INT_SRC_INTPHC11, 
  *    CG_INT_SRC_INTPHC1EVRY.
  * @retval None
  */
void CG_ClearINTReq(CG_INTSrc INTSource)
{
    uint32_t regval = INTSource;

    /* Check the parameters */
    assert_param(IS_CG_INT_SRC(INTSource));

    TSB_CG->ICRCG = regval;
}

/**
  * @brief  Get the type of reset reason and clear the reset flag.
  * @param  None
  * @retval Reset flag
  */
CG_ResetFlag CG_GetResetFlag(void)
{
    CG_ResetFlag reset_flag = { 0U };

    reset_flag.All = TSB_CG->RSTFLG & CG_RESET_FLAG_MASK;
    TSB_CG->RSTFLG = 0U;

    return reset_flag;
}

/**
  * @brief  Enable or disable supplying clock Fc for PSC and ADC(unit A, B, C)
  * @param  Periph: The target peripheral that CG supplies fc for
  *   This parameter can be one of the following values or their combination:
  *     CG_PSC_CLK_SUPPLY: PSC clock supply
  *     CG_ADC_C_CLK_SUPPLY: ADC unit C clock supply
  *     CG_ADC_B_CLK_SUPPLY: ADC unit B clock supply
  *     CG_ADC_A_CLK_SUPPLY: ADC unit A clock supply    
  * @param  NewState: New state of clock Fc supply setting.
  *   This parameter can be one of the following values:
  *     DISABLE or ENABLE
  * @retval None
  */
void CG_SetPeriphClkSupply(uint32_t Periph, FunctionalState NewState)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_CG_PERIPH_CLK_SUPPLY(Periph));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {   /* clear to '0' to enable */
        TSB_CG->SYSCR &= (~Periph);
    } else {
        if (Periph == CG_PSC_CLK_SUPPLY) {
            /* Do nothing */
        } else {
            do {                /* wait that ADC module is stop: MOD5 bit1,3 is '1' */
                tmp = TSB_ADA->MOD5;
                tmp &= TSB_ADB->MOD5;
                tmp &= TSB_ADC->MOD5;
            } while ( /*tmp != 0x0AU */ 0);
        }
        TSB_CG->SYSCR |= Periph;        /* write '1' to enable */
    }
}

/**
  * @brief  Enable or disable supplying clock fclk to modules group A
  * @param  Periph: The target peripheral that CG supplies clock fclk for
  *   This parameter can be one of the following values or their combination:
  *     FCLK_A_TMRB10, 
  *     FCLK_A_TMRB11, 
  *     FCLK_A_TMRB12, 
  *     FCLK_A_TMRB13, 
  *     FCLK_A_TMRB14, 
  *     FCLK_A_TMRB15, 
  *     FCLK_A_TMRB16, 
  *     FCLK_A_TMRB17, 
  *     FCLK_A_TMRB18, 
  *     FCLK_A_TMRB19, 
  *     FCLK_A_DAC0  ,
  *     FCLK_A_DAC1  ,
  *     FCLK_A_EBIF  ,
  *     FCLK_A_UART0 ,
  *     FCLK_A_UART1 ,
  *     FCLK_A_DMACA ,
  *     FCLK_A_DMACB ,
  *     FCLK_A_DMACC ,
  *     FCLK_A_PORTA ,
  *     FCLK_A_PORTB ,
  *     FCLK_A_PORTC ,
  *     FCLK_A_PORTD ,
  *     FCLK_A_PORTE ,
  *     FCLK_A_PORTF ,
  *     FCLK_A_PORTG ,
  *     FCLK_A_PORTH ,
  *     FCLK_A_PORTJ ,
  *     FCLK_A_PORTK ,
  *     FCLK_A_PORTL ,
  *     FCLK_A_PORTM ,
  *     FCLK_A_PORTN ,
  *     FCLK_A_ALL    
  * @param  NewState: New state of clock supply setting.
  *   This parameter can be one of the following values:
  *     DISABLE or ENABLE
  * @retval None
  */
void CG_SetFclkPeriphA(uint32_t Periph, FunctionalState NewState)
{
    assert_param(IS_CG_FCLK_SUPPLYA(Periph));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    if (NewState == ENABLE) {   /* clear to '0' to enable */
        TSB_CG->FCLKMSKA &= (uint32_t) (~((uint32_t) Periph));
    } else {                    /* write '1' to disable   */
        TSB_CG->FCLKMSKA |= (uint32_t) (Periph);
    }
}

/**
  * @brief  Enable or disable supplying clock fclk to modules group B
  * @param  Periph: The target peripheral that CG supplies clock fclk for
  *   This parameter can be one of the following values or their combination:
  *     FCLK_B_PORTP , 
  *     FCLK_B_PORTR , 
  *     FCLK_B_PORTT , 
  *     FCLK_B_PORTU , 
  *     FCLK_B_PORTV , 
  *     FCLK_B_PORTW , 
  *     FCLK_B_PORTY , 
  *     FCLK_B_PORTAA, 
  *     FCLK_B_PORTAB, 
  *     FCLK_B_PORTAC, 
  *     FCLK_B_PORTAD,
  *     FCLK_B_PORTAE,
  *     FCLK_B_PORTAF,
  *     FCLK_B_PORTAG,
  *     FCLK_B_PORTAH,
  *     FCLK_B_PORTAJ,
  *     FCLK_B_ADCA  ,
  *     FCLK_B_ADCB  ,
  *     FCLK_B_ADCC  ,
  *     FCLK_B_EPHC  ,
  *     FCLK_B_SBI   ,
  *     FCLK_B_WDT   ,
  *     FCLK_B_ALL 
  * @param  NewState: New state of clock supply setting.
  *   This parameter can be one of the following values:
  *     DISABLE or ENABLE
  * @retval None
  */
void CG_SetFclkPeriphB(uint32_t Periph, FunctionalState NewState)
{
    assert_param(IS_CG_FCLK_SUPPLYB(Periph));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    if (NewState == ENABLE) {   /* clear to '0' to enable */
        TSB_CG->FCLKMSKB &= (uint32_t) (~((uint32_t) Periph));
    } else {                    /* write '1' to disable   */
        TSB_CG->FCLKMSKB |= (uint32_t) (Periph);
    }
}


/**
  * @brief  Enable or disable supplying clock fc to modules group A
  * @param  Periph: The target peripheral that CG supplies clock fc for
  *   This parameter can be one of the following values or their combination:
  *     FC_A_ESIO0      ,
  *     FC_A_ESIO1      ,
  *     FC_A_ESIO2      ,
  *     FC_A_TMRD       ,
  *     FC_A_SIO_UART0  ,
  *     FC_A_SIO_UART1  ,
  *     FC_A_SIO_UART2  ,
  *     FC_A_SIO_UART3  ,
  *     FC_A_SIO_UART4  ,
  *     FC_A_SIO_UART5  ,
  *     FC_A_TMRB00     ,
  *     FC_A_TMRB01     ,
  *     FC_A_TMRB02     ,
  *     FC_A_TMRB03     ,
  *     FC_A_TMRB04     ,
  *     FC_A_TMRB05     ,
  *     FC_A_TMRB06     ,
  *     FC_A_TMRB07     ,
  *     FC_A_TMRB08     ,
  *     FC_A_TMRB09     ,
  *     FC_A_TMRCCAP0   ,
  *     FC_A_TMRCCAP1   ,
  *     FC_A_TMRCCAP2   ,
  *     FC_A_TMRCCAP3   ,
  *     FC_A_TMRCCMP0   ,
  *     FC_A_TMRCCMP1   ,
  *     FC_A_TMRCCMP2   ,
  *     FC_A_TMRCCMP3   ,
  *     FC_A_TMRCCMP4   ,
  *     FC_A_TMRCCMP5   ,
  *     FC_A_TMRCCMP6   ,
  *     FC_A_TMRCCMP7   ,
  *     FC_A_ALL         
  * @param  NewState: New state of clock supply setting.
  *   This parameter can be one of the following values:
  *     DISABLE or ENABLE
  * @retval None
  */
void CG_SetFcPeriphA(uint32_t Periph, FunctionalState NewState)
{
    /* assert_param(IS_CG_FC_SUPPLYA(Periph)); */
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    if (NewState == ENABLE) {   /* clear to '0' to enable */
        TSB_CG->FCMSKA &= (uint32_t) (~((uint32_t) Periph));
    } else {                    /* write '1' to disable   */
        TSB_CG->FCMSKA |= (uint32_t) (Periph);
    }
}



/**
  * @brief  Enable or disable supplying clock fc to modules group B
  * @param  Periph: The target peripheral that CG supplies clock fc for
  *   This parameter can be one of the following values or their combination:
  *     FC_B_TMRCTBT      ,
  * @param  NewState: New state of clock supply setting.
  *   This parameter can be one of the following values:
  *     DISABLE or ENABLE
  * @retval None
  */
void CG_SetFcPeriphB(uint32_t Periph, FunctionalState NewState)
{
    assert_param(IS_CG_FC_SUPPLYB(Periph));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    if (NewState == ENABLE) {   /* clear to '0' to enable */
        TSB_CG->FCMSKB &= (uint32_t) (~((uint32_t) Periph));
    } else {                    /* write '1' to disable   */
        TSB_CG->FCMSKB |= (uint32_t) (Periph);
    }
}



/** @} */
/* End of group CG_Exported_Functions */

/** @} */
/* End of group CG */

/** @} */
/* End of group TX04_Periph_Driver */
#endif                          /* defined(__TMPM440_CG_H) */
