/**
 *******************************************************************************
 * @file    tmpm440_esio.c
 * @brief   This file provides API functions for ESIO driver.
 * @version V2.0.2.3
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_esio.h"

#if defined(__TMPM440_ESIO_H)

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup ESIO 
  * @brief ESIO driver modules
  * @{
  */

/** @defgroup ESIO_Private_Defines
  * @{
  */

#define ESIO_CR2FIXWRITE    (uint32_t)(0x01 << 16U)     /* bit 16 of CR2 must be wrote '1' */




/** @} */
/* End of group ESIO_Private_Defines */

/** @defgroup ESIO_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group ESIO_Private_FunctionPrototypes */

/** @defgroup ESIO_Private_Functions
  * @{
  */

/** @} */
/* End of group ESIO_Private_Functions */

/** @defgroup ESIO_Exported_Functions
  * @{
  */

/**
  * @brief Software reset ESIO module.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval None.
  */
void ESIO_SWReset(TSB_ESIO_TypeDef * ESIOx)
{
    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    /* write '10' then '01' to bit7-6 */
    ESIOx->CR0 = (uint32_t) (0x02U << 6U);
    ESIOx->CR0 = (uint32_t) (0x01U << 6U);
}

/**
  * @brief Enable ESIO module, it must be called at first.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval None.
  */
void ESIO_Enable(TSB_ESIO_TypeDef * ESIOx)
{
    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    /* write '1' to bit 0, no need to care about bit7-6 */
    ESIOx->CR0 = 0x01U;
}


/**
  * @brief Disable ESIO module.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval None.
  */
void ESIO_Disable(TSB_ESIO_TypeDef * ESIOx)
{
    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    /* write '0' to bit 0 */
    ESIOx->CR0 = 0x0U;
}

/**
  * @brief Enable/Disable ESIO Transfer.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param NewState: Specify the state for transfer.
  *   This parameter can be one of the following values:
  *    ENABLE or DISABLE.  
  * @retval None.
  */
void ESIO_SetTxRxCtrl(TSB_ESIO_TypeDef * ESIOx, FunctionalState NewState)
{
    /* CR1 used: bit14, 13, 11-10, 8, 7-0, clear bit14 here */
    uint32_t tmp = ESIOx->CR1 & ((0x01U << 13U) | (0x03U << 10U) | (0x01U << 8U) | (0xFFU));

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* write '1' to bit 14 to enable, write '0' to disable */
    tmp |= (uint32_t) (NewState) << 14U;
    ESIOx->CR1 = tmp;
}

/**
  * @brief Get the ESIO transfer control status.
  * @param ESIOx:  Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval The transfer control status of specify ESIO unit.
  *   This value can be one of the following values:
  *    ENABLE or DISABLE.   
  */
FunctionalState ESIO_GetTxRxCtrl(TSB_ESIO_TypeDef * ESIOx)
{
    /* CR1 used: bit14, 13, 11-10, 8, 7-0, only keep bit14 here */
    uint32_t tmp = ESIOx->CR1 & (0x01U << 14U);

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    return (FunctionalState) (tmp >> 14U);
}


/**
  * @brief Select ESIO mode: as SIO or SPI.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param Mode: Specify the mode for ESIO.
  *   This parameter can be one of the following values:
  *    ESIO_MODE_SPI or ESIO_MODE_SIO.  
  * @retval None.  
  */
void ESIO_SelectMode(TSB_ESIO_TypeDef * ESIOx, ESIO_Mode Mode)
{
    /* CR1 used: bit14, 13, 11-10, 8, 7-0, clear bit13 here */
    uint32_t tmp = ESIOx->CR1 & ((0x01U << 14U) | (0x03U << 10U) | (0x01U << 8U) | (0xFFU));

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_MODE(Mode));

    /* write '0' to bit 13 for SPI, write '1' for SIO */
    tmp |= (uint32_t) (Mode) << 13U;
    ESIOx->CR1 = tmp;
}



/**
  * @brief Select ESIO transfer mode
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param TrMode: Specify the transfer mode for ESIO.
  *   This parameter can be one of the following values:
  *    ESIO_TRMODE_TX, ESIO_TRMODE_RX or ESIO_TRMODE_TXRX.  
  * @retval None.  
  */
void ESIO_SelectTransferMode(TSB_ESIO_TypeDef * ESIOx, ESIO_TransferMode TrMode)
{
    /* CR1 used: bit14, 13, 11-10, 8, 7-0, clear bit11-10 here */
    uint32_t tmp = ESIOx->CR1 & ((0x01U << 14U) | (0x01U << 13U) | (0x01U << 8U) | (0xFFU));

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_TRANSFERMODE(TrMode));

    /* write 'TrMode' to bit11-10 */
    tmp |= (uint32_t) (TrMode) << 10U;
    ESIOx->CR1 = tmp;
}


/**
  * @brief Select chip selection pin.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param CSx: Specify the chip selection pin.
  *   This parameter can be one of the following values:
  *    ESIO_CS0 or ESIO_CS1.  
  * @retval None.  
  */
void ESIO_SetCS(TSB_ESIO_TypeDef * ESIOx, ESIO_CSx CSx)
{
    /* CR1 used: bit14, 13, 11-10, 8, 7-0, clear bit8 here */
    uint32_t tmp = ESIOx->CR1 & ((0x01U << 14U) | (0x01U << 13U) | (0x03U << 10U) | (0xFFU));

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_CSX(CSx));

    /* write 'CSx' to bit8 */
    tmp |= (uint32_t) (CSx) << 8U;
    ESIOx->CR1 = tmp;
}

/**
  * @brief Set the number of transfer frame. 
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param Num: Specify the number of transferring.
  *   This parameter can be one of the following values:
  *    1: One frame (Single transfer)  
  *    2-255: 2 to 255 frames (Burst transfer)  
  * @retval None.  
  */
void ESIO_SetTransferNum(TSB_ESIO_TypeDef * ESIOx, uint8_t Num)
{
    /* CR1 used: bit14, 13, 11-10, 8, 7-0, clear bit7-0 here */
    uint32_t tmp = ESIOx->CR1 & ((0x01U << 14U) | (0x01U << 13U) | (0x03U << 10U) | (0x01U << 8U));

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_TRANSFERNUM(Num));

    /* write 'Num' to bit7-0 */
    tmp |= (uint32_t) (Num);
    ESIOx->CR1 = tmp;
}

/**
  * @brief Set the status of ESIOxTXD pin during idle cycle 
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param PinMode: Specify the mode of ESIOxTXD pin.
  *   This parameter can be one of the following values:
  *    ESIO_TXPIN_FINALBIT, ESIO_TXPIN_KEEPLOW, ESIO_TXPIN_KEEPHIGH  
  * @retval None.  
  */
void ESIO_SetTxPinInIdle(TSB_ESIO_TypeDef * ESIOx, ESIO_TxPinInIdle PinMode)
{
    /* CR2 used: bit23-22, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit 23, 22 here */
    uint32_t tmp = ESIOx->CR2 & (0xFFF7U);

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_TXPININIDLE(PinMode));

    /* write 'PinMode' to bit23-22 */
    tmp |= (uint32_t) (PinMode) << 22U;
    ESIOx->CR2 = tmp | ESIO_CR2FIXWRITE;
}

/**
  * @brief Set the FIFO fill level to generate transfer(Tx or Rx) interrupt. 
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param TxRx: Specify tranmit or receive.
  *   This parameter can be one of the following values:
  *    ESIO_TX, ESIO_RX
  * @param Level: Specify the FIFO fill level to generate interrupt.
  *   This parameter can be one of the following values:
  *    0 to 8  (depended on the number of LINEs and the length of frame)
  * @retval None.  
  */
void ESIO_SetFIFOLevelINT(TSB_ESIO_TypeDef * ESIOx, uint8_t TxRx, uint8_t Level)
{
    /* read CR2 */
    uint32_t tmp = ESIOx->CR2;

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_TXRX(TxRx));
    assert_param(IS_ESIO_FIFOLEVELINT(Level));

    if (TxRx == ESIO_TX) {
        /* CR2 used: bit23-22, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit15-12 here */
        tmp &= 0x00C00FF7U;
        tmp |= (uint32_t) Level << 12U;
    } else {                    /* is ESIO_RX */
        /* CR2 used: bit23-22, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit11-8 here */
        tmp &= 0x00C0F0F7U;
        tmp |= (uint32_t) Level << 8U;
    }

    ESIOx->CR2 = tmp | ESIO_CR2FIXWRITE;
}

/**
  * @brief Set Transfer DMA request control. 
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param TxRx: Specify tranmit or receive.
  *   This parameter can be one of the following values:
  *    ESIO_TX, ESIO_RX
  * @param NewState: Specify the state of transfer DMA request.
  *   This parameter can be one of the following values:
  *    ENABLE or DISABLE. 
  * @retval None.  
  */
void ESIO_SetDMA(TSB_ESIO_TypeDef * ESIOx, uint8_t TxRx, FunctionalState NewState)
{
    /* read CR2 */
    uint32_t tmp = ESIOx->CR2;

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_TXRX(TxRx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (TxRx == ESIO_TX) {
        /* CR2 used: bit23-22, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit1 here */
        tmp &= 0x00C0FFF5U;
        tmp |= (uint32_t) (NewState) << 1U;
    } else {                    /* is ESIO_RX */
        /* CR2 used: bit23-22, 15-12, 11-8, 7,6,5,4, 2,1,0.  clear bit0 here */
        tmp &= 0x00C0FFF6U;
        tmp |= (uint32_t) (NewState);
    }

    ESIOx->CR2 = tmp | ESIO_CR2FIXWRITE;
}


/**
  * @brief Enable/Disable the interrupt source
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param IntSrc: Specify the interrupt source to be set
  *   This parameter can be one of the following values or their combination:
  *    ESIO_INT_PERR, 
  *    ESIO_INT_RXEND, ESIO_INT_RXFIFO,
  *    ESIO_INT_TXEND, ESIO_INT_TXFIFO, 
  *    ESIO_INT_ALL (all above interrupt source)    
  * @param NewState: Specify the state for interrupt control.
  *   This parameter can be one of the following values:
  *    ENABLE or DISABLE.   
  * @retval None.  
  */
void ESIO_SetINT(TSB_ESIO_TypeDef * ESIOx, uint32_t IntSrc, FunctionalState NewState)
{
    /* CR2 used: bit23-22, 15-12, 11-8, 7,6,5,4, 2,1,0. */
    uint32_t tmp = ESIOx->CR2 & 0x00C0FFF7U;

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_INTSRC(IntSrc));

    if (NewState == ENABLE) {
        /* write 'IntSrc' to bit7,6,5,4, 2 */
        tmp |= IntSrc;
    } else {
        tmp &= ~IntSrc;
    }
    ESIOx->CR2 = tmp | ESIO_CR2FIXWRITE;
}


/**
  * @brief Initialize transfer FIFO pointer. 
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param TxRx: Specify tranmit or receive.
  *   This parameter can be one of the following values:
  *    ESIO_TX, ESIO_RX
  * @retval None.  
  */
void ESIO_InitFIFO(TSB_ESIO_TypeDef * ESIOx, uint8_t TxRx)
{
    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_TXRX(TxRx));

    if (TxRx == ESIO_TX) {
        ESIOx->CR3 = 0x02U;
    } else {
        ESIOx->CR3 = 0x01U;
    }
}


/**
  * @brief Set the clock and divider for baud rate generator
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param Clk: Specify the input clock to the baud rate generator.
  *   This parameter can be one of the following values:
  *    ESIO_PHIT0_DIVIDE_2,     ESIO_PHIT0_DIVIDE_4
  *    ESIO_PHIT0_DIVIDE_8,     ESIO_PHIT0_DIVIDE_16  
  *    ESIO_PHIT0_DIVIDE_32,    ESIO_PHIT0_DIVIDE_64
  *    ESIO_PHIT0_DIVIDE_128,   ESIO_PHIT0_DIVIDE_256 
  *    ESIO_PHIT0_DIVIDE_512,   ESIO_PHIT0_DIVIDE_1024
  * @param Divider: Specify the division ratio "N" 
  *                 (Note: clock output to pin ESIOxSCK = Clk / N ).
  *   This parameter can be one of the following values:
  *     1 to 16
  * @retval None.  
  */
void ESIO_SetBaudRate(TSB_ESIO_TypeDef * ESIOx, ESIO_BaudClock Clk, uint8_t Divider)
{
    uint32_t tmp = Divider;

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_BAUDCLOCK(Clk, ((uint32_t) Divider)));
    assert_param(IS_ESIO_DIVIDER(Divider));

    if (tmp == 16U) {
        tmp = 0U;
    } else {
        /* Do nothing */
    }
    ESIOx->BR = (((uint32_t) Clk) << 4U) | (tmp);
}

/**
  * @brief Initialize the specified ESIO channel 
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param Init: The structure containing basic ESIO configuration.
  * @retval None.  
  * NOTE: call ESIO_SetTransferNum() at first.
  */
void ESIO_Init(TSB_ESIO_TypeDef * ESIOx, ESIO_InitTypeDef * Init)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_DATADIRECTION(Init->DataDirection));
    assert_param(IS_ESIO_FRAMELENGTH(Init->FrameLength));
    assert_param(IS_ESIO_CYCLEBETWEENFRAMES(Init->CycleBetweenFrames));
    assert_param(IS_ESIO_NUMBEROFLINE(Init->NumberOfLINE));
    assert_param(IS_ESIO_CLKPOLORITY(Init->ClkPolority));
    assert_param(IS_ESIO_SHORTESTIDLECYCLE(Init->ShortestIdleCycle));
    assert_param(IS_ESIO_CSACTIVELEVEL(Init->CS1ActiveLevel));
    assert_param(IS_ESIO_CSACTIVELEVEL(Init->CS0ActiveLevel));
    assert_param(IS_ESIO_DELAYCYCLENUM(Init->DelayCycleNum_CS_SCLK));
    assert_param(IS_ESIO_DELAYCYCLENUM(Init->DelayCycleNum_Negate_CS));
    assert_param(IS_FUNCTIONAL_STATE(Init->HorizontalParityCheck));
    assert_param(IS_ESIO_PARITYTYPE(Init->HorizontalParity));
    assert_param(IS_FUNCTIONAL_STATE(Init->VerticalParityCheck));
    assert_param(IS_ESIO_PARITYTYPE(Init->VerticalParity));

    /* Horizontal parity must be disabled in single transfer. */
    assert_param(IS_ESIO_PARITYINSINGLE(ESIOx->CR1, Init->HorizontalParityCheck));


    tmp = (uint32_t) Init->DataDirection << 31U;
    tmp |= (uint32_t) Init->FrameLength << 24U;
    tmp |= (uint32_t) Init->CycleBetweenFrames << 20U;
    tmp |= ((uint32_t) Init->NumberOfLINE - 1U) << 16U;
    tmp |= (uint32_t) Init->ClkPolority << 14U;
    tmp |= (uint32_t) Init->ShortestIdleCycle << 10U;
    tmp |= (uint32_t) Init->CS1ActiveLevel << 9U;
    tmp |= (uint32_t) Init->CS0ActiveLevel << 8U;
    tmp |= ((uint32_t) Init->DelayCycleNum_CS_SCLK - 1U) << 4U;
    tmp |= ((uint32_t) Init->DelayCycleNum_Negate_CS - 1U) << 0U;
    ESIOx->FMTR0 = tmp;

    tmp = (uint32_t) Init->HorizontalParityCheck << 3U;
    tmp |= (uint32_t) Init->HorizontalParity << 2U;
    tmp |= (uint32_t) Init->VerticalParityCheck << 1U;
    tmp |= (uint32_t) Init->VerticalParity << 0U;
    ESIOx->FMTR1 = tmp;

}

/**
  * @brief Write data to transmit FIFO 
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param dat: The data which will be sent.
  * @retval None.  
  */
void ESIO_SetTxData(TSB_ESIO_TypeDef * ESIOx, uint32_t Dat)
{
    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    /* Do not write data to 'DR' register when transmit FIFO is full */
    ESIOx->DR = Dat;
}

/**
  * @brief Read data from receive FIFO 
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval The received data.  
  */
uint32_t ESIO_GetRxData(TSB_ESIO_TypeDef * ESIOx)
{
    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    /* Do not read data from 'DR' register when receive FIFO is empty */
    return ESIOx->DR;
}


/**
  * @brief Check if ESIO registers is modifiable.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval The modifiable status of ESIO register. 
  *   This parameter can be one of the following values:
  *    ENABLE or DISABLE.    
  */
FunctionalState ESIO_IsRegisterModifiable(TSB_ESIO_TypeDef * ESIOx)
{
    FunctionalState tmp = ENABLE;

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    if (ESIOx->SR & (0x01U << 31U)) {   /* bit31 == 1: cann't be modified */
        tmp = DISABLE;
    } else {
        /* Do nothing */
    }
    return tmp;

}


/**
  * @brief Get the ESIO status.
  * @param ESIOx:  Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval The Tx/Rx FIFO, Run, Interrupt status of ESIO unit.
  */
ESIO_StatusFlag ESIO_GetStatus(TSB_ESIO_TypeDef * ESIOx)
{
    ESIO_StatusFlag tmp = { 0U };

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    /* the unused bit is read as '0', no need to mask them */
    tmp.All = ESIOx->SR;
    return tmp;
}


/**
  * @brief Get the ESIO error information.
  * @param ESIOx:  Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval The Error number for LINEy and Horizontal/Vertical error status
  */
ESIO_ParityErrNum ESIO_GetParityErrorFlag(TSB_ESIO_TypeDef * ESIOx)
{
    ESIO_ParityErrNum tmp = { 0U };

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    /* the unused bit is read as '0', no need to mask them */
    /* memer LINEy_ParityErrNum large than 2 means Parity error is occurred twice or more times. */
    tmp.All = ESIOx->PERR;
    return tmp;
}


/**
  * @brief Clear all ESIO error flag.
  * @param ESIOx:  Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval None
  */
void ESIO_ClrAllParityErrFlag(TSB_ESIO_TypeDef * ESIOx)
{

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    /* bit01,23,45,67 can only be wrote as '0b11', so write them together */
    ESIOx->PERR = (uint32_t) 0x3FF;
}


/**
  * @brief Get the ESIO horizontal error information.
  * @param ESIOx:  Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param Line: Select the LINE number.
  *   This parameter can be one of the following values:
  *    ESIO_LINE0, ESIO_LINE1, 
  *    ESIO_LINE2, ESIO_LINE3   
  * @retval The horizontal error infomation for LINEy 
  */
uint32_t ESIO_GetHorizontalParityError(TSB_ESIO_TypeDef * ESIOx, ESIO_LINEx Line)
{
    uint32_t tmpAddr = (uint32_t) (&ESIOx->HPERR0);

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_LINEx(Line));

    tmpAddr += (0x4U * (uint32_t) Line);
    return (*((uint32_t *) (tmpAddr)));
}


/**
  * @brief Clear the ESIO horizontal error information.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @param Line: Select the LINE number.
  *   This parameter can be one of the following values:
  *    ESIO_LINE0, ESIO_LINE1, 
  *    ESIO_LINE2, ESIO_LINE3   
  * @retval none
  */
void ESIO_ClrHorizontalParityError(TSB_ESIO_TypeDef * ESIOx, ESIO_LINEx Line)
{
    uint32_t tmpAddr = (uint32_t) (&ESIOx->HPERR0);

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    assert_param(IS_ESIO_LINEx(Line));

    tmpAddr += (0x4U * (uint32_t) Line);
    *((uint32_t *) (tmpAddr)) = 0xFFFFFFFFU;
}

/**
  * @brief Get the ESIO vertical error information.
  * @param ESIOx:  Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval The vertical error infomation for LINEy 
  */
ESIO_VerticalParityErrFrame ESIO_GetVerticalParityErrFrame(TSB_ESIO_TypeDef * ESIOx)
{
    ESIO_VerticalParityErrFrame tmp = { 0U };

    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));

    tmp.All = ESIOx->VPERR;
    return tmp;
}


/**
  * @brief Clear the ESIO vertical error information.
  * @param ESIOx: Select ESIO unit.
  *   This parameter can be one of the following values:
  *    TSB_ESIO0, TSB_ESIO1, TSB_ESIO2
  * @retval none
  */
void ESIO_ClrAllVerticalParityErrNum(TSB_ESIO_TypeDef * ESIOx)
{
    /* Check the parameters */
    assert_param(IS_ESIO_UNIT(ESIOx));
    ESIOx->VPERR = 0xFFFFFFFFU;
}


/** @} */
/* End of group ESIO_Exported_Functions */

/** @} */
/* End of group ESIO */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_ESIO_H) */
