/**
 *******************************************************************************
 * @file    tmpm440_exb.c
 * @brief   This file provides API functions for EXB driver.
 * @version V2.0.2.4
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_exb.h"

#if defined(__TMPM440_EXB_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup EXB 
  * @brief EXB driver modules
  * @{
  */
/** @defgroup EXB_Private_Defines
  * @{
  */
#define MODE_EXBWAIT_MASK    ((uint32_t)0x00000001U)
#define EXB_AS_FIXED_BITS    ((uint32_t)0x60000000U)
#define CSX_CSW0_CLEAR       ((uint32_t)0xFFFFFFFEU)
#define CSX_CSW0_SET         ((uint32_t)0x00000001U)
#define ENDTYPE_CLEAR        ((uint32_t)0xFFFFFF7FU)
#define ENDTYPE_SET          ((uint32_t)0x00000080U)
#define SEPARATE_BUSMODE     ((uint32_t)0x00000001U)
#define MULTIPLEX_BUSMODE    ((uint32_t)0xFFFFFFFEU)
/** @} */
/* End of group EXB_Private_Defines */

/** @defgroup EXB_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group EXB_Private_FunctionPrototypes */

/** @defgroup EXB_Private_Functions
  * @{
  */

/** @} */
/* End of group group EXB_Private_Functions */

/** @defgroup EXB_Exported_Functions
  * @{
  */

/**
  * @brief  Set EXB bus mode
  * @param  BusMode: Select EXB mode.
  *         This parameter can be: EXB_BUS_SEPARATE or EXB_BUS_MULTIPLEX.
  * @retval None
  */
void EXB_SetBusMode(uint8_t BusMode)
{
    /* Check the parameters */
    assert_param(IS_EXB_BUS_MODE(BusMode));

    /* Set EXBMOD<EXBSEL> to select bus mode. */
    if (BusMode == EXB_BUS_SEPARATE) {
        TSB_EXB->MOD |= SEPARATE_BUSMODE;
    } else {
        TSB_EXB->MOD &= MULTIPLEX_BUSMODE;
    }
}

/**
  * @brief  Set the setup, wait and recovery of the bus cycle to be double or quadruple
  * @param  Cycle: Set the bus cycle to be double or quadruple.
  *         This parameter can be:
  *         EXB_CYCLE_NONE, EXB_CYCLE_DOUBLE or EXB_CYCLE_QUADRUPLE
  * @retval None
  */
void EXB_SetBusCycleExtension(uint8_t Cycle)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_EXB_BUS_CYCLE_EXTENSION(Cycle));

    tmp = TSB_EXB->MOD;
    tmp &= MODE_EXBWAIT_MASK;
    tmp |= (uint32_t) Cycle;
    /* Set EXBMOD<EXBWAIT1:0> to set bus cycle extension. */
    TSB_EXB->MOD = tmp;
}

/**
  * @brief  Set the endian type for external memory or peropheral I/O
  * @param  ChipSelect: Select Chip
  *         This parameter can be one of the following values:
  *         EXB_CS0, EXB_CS1.  
  * @param  EndianType: classifier a type
  *         This parameter can be one of the following values:
  *         SAME_ENDIAN_AS_CPU, NOT_SAME_ENDIAN_AS_CPU.
  * @retval None
  */
void EXB_SetEndianType(uint8_t ChipSelect, EXB_EndianType EndianType)
{
    /* Check the parameters */
    assert_param(IS_EXB_CHIP_SELECT(ChipSelect));
    assert_param(IS_EXB_ENDIAN_TYPE(EndianType));

    switch (ChipSelect) {
    case EXB_CS0:
        TSB_EXB->CS0 =(EndianType ==SAME_ENDIAN_AS_CPU) ? (TSB_EXB->CS0 & ENDTYPE_CLEAR) : (TSB_EXB->CS0 | ENDTYPE_SET);
        break;

    case EXB_CS1:
        TSB_EXB->CS1 =(EndianType ==SAME_ENDIAN_AS_CPU) ? (TSB_EXB->CS1 & ENDTYPE_CLEAR) : (TSB_EXB->CS1 | ENDTYPE_SET);
        break;

    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Enable the specified chip
  * @param  ChipSelect: Select Chip
  *         This parameter can be one of the following values:
  *         EXB_CS0, EXB_CS1.
  * @retval None
  */
void EXB_Enable(uint8_t ChipSelect)
{
    /* Check the parameters */
    assert_param(IS_EXB_CHIP_SELECT(ChipSelect));

    switch (ChipSelect) {
    case EXB_CS0:
        TSB_EXB->CS0 |= CSX_CSW0_SET;
        break;

    case EXB_CS1:
        TSB_EXB->CS1 |= CSX_CSW0_SET;
        break;

    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Disable the specified chip
  * @param  ChipSelect: Select Chip
  *         This parameter can be one of the following values:
  *         EXB_CS0, EXB_CS1.
  * @retval None
  */

void EXB_Disable(uint8_t ChipSelect)
{
    /* Check the parameters */
    assert_param(IS_EXB_CHIP_SELECT(ChipSelect));

    switch (ChipSelect) {
    case EXB_CS0:
        TSB_EXB->CS0 &= CSX_CSW0_CLEAR;
        break;

    case EXB_CS1:
        TSB_EXB->CS1 &= CSX_CSW0_CLEAR;
        break;

    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Initialize the specified chip
  * @param  ChipSelect: Select Chip
  *         This parameter can be one of the following values:
  *         EXB_CS0, EXB_CS1.
  * @param  InitStruct: The structure containing basic EXB configuration.
  *         This parameter is structure pointer which consists of following members:
  *         AddrSpaceSize, StartAddr, BusWidth and Cycles
  * @retval None
  */
void EXB_Init(uint8_t ChipSelect, EXB_InitTypeDef * InitStruct)
{
    uint32_t tmp_AS = 0U;
    uint32_t tmp_CS = 0U;
    /* Check the parameters */
    assert_param(IS_EXB_CHIP_SELECT(ChipSelect));
    assert_param(IS_POINTER_NOT_NULL(InitStruct));
    assert_param(IS_EXB_ADDR_SPACE_SIZE(InitStruct->AddrSpaceSize));
    assert_param(IS_EXB_STARTADDR_VALID(InitStruct->StartAddr));
    assert_param(IS_EXB_ADDR_VALID(InitStruct->StartAddr, InitStruct->AddrSpaceSize));
    assert_param(IS_EXB_DATA_BUS_WIDTH(InitStruct->BusWidth));
    assert_param(IS_EXB_INTERNAL_WAIT(InitStruct->Cycles.InternalWait));
    assert_param(IS_EXB_RW_SETUP_CYCLE(InitStruct->Cycles.ReadSetupCycle));
    assert_param(IS_EXB_RW_SETUP_CYCLE(InitStruct->Cycles.WriteSetupCycle));
    assert_param(IS_EXB_ALE_WAIT_CYCLE(InitStruct->Cycles.ALEWaitCycle));
    assert_param(IS_EXB_RW_RECOVERY_CYCLE(InitStruct->Cycles.ReadRecoveryCycle));
    assert_param(IS_EXB_RW_RECOVERY_CYCLE(InitStruct->Cycles.WriteRecoveryCycle));
    assert_param(IS_EXB_CS_RECOVERY_CYCLE(InitStruct->Cycles.ChipSelectRecoveryCycle));

    /* Initialize EXB ASx */
    tmp_AS = (uint32_t) InitStruct->AddrSpaceSize | (((uint32_t) InitStruct->StartAddr) << 16U);
    tmp_AS |= EXB_AS_FIXED_BITS;

    /* Initialize EXB CSx */
    tmp_CS = ((uint32_t) InitStruct->Cycles.InternalWait << 8U) |
        ((uint32_t) InitStruct->Cycles.ReadRecoveryCycle << 24U) |
        ((uint32_t) InitStruct->Cycles.WriteRecoveryCycle << 27U);
    tmp_CS |= (InitStruct->Cycles.ReadSetupCycle == EXB_CYCLE_4) ?
        ((uint32_t) (InitStruct->Cycles.ReadSetupCycle - 1UL) << 16U) :
        ((uint32_t) InitStruct->Cycles.ReadSetupCycle << 16U);
    tmp_CS |= (InitStruct->Cycles.WriteSetupCycle == EXB_CYCLE_4) ?
        ((uint32_t) (InitStruct->Cycles.WriteSetupCycle - 1UL) << 18U) :
        ((uint32_t) InitStruct->Cycles.WriteSetupCycle << 18U);

    if ((TSB_EXB->MOD & SEPARATE_BUSMODE) == 0U) {
        /* Set ALE wait cycle for multiplex bus */
        tmp_CS |= (InitStruct->Cycles.ALEWaitCycle == EXB_CYCLE_4) ?
            ((uint32_t) (InitStruct->Cycles.ALEWaitCycle - 1UL) << 20U) :
            ((uint32_t) InitStruct->Cycles.ALEWaitCycle << 20U);
    } else {
        /* No need ALE wait cycle for separate bus */
        InitStruct->Cycles.ALEWaitCycle = EXB_CYCLE_0;
        tmp_CS |= (uint32_t) InitStruct->Cycles.ALEWaitCycle << 20U;
    }

    tmp_CS |= (InitStruct->Cycles.ChipSelectRecoveryCycle == EXB_CYCLE_4) ?
        ((uint32_t) (InitStruct->Cycles.ChipSelectRecoveryCycle - 1UL) << 30U) :
        ((uint32_t) InitStruct->Cycles.ChipSelectRecoveryCycle << 30U);
    tmp_CS |= (uint32_t) InitStruct->BusWidth;


    switch (ChipSelect) {
    case EXB_CS0:
        TSB_EXB->AS0 = tmp_AS;
        TSB_EXB->CS0 = tmp_CS;
        break;

    case EXB_CS1:
        TSB_EXB->AS1 = tmp_AS;
        TSB_EXB->CS1 = tmp_CS;
        break;

    default:
        /* Do nothing */
        break;
    }
}

/** @} */
/* End of group EXB_Exported_Functions */

/** @} */
/* End of group EXB */

/** @} */
/* End of group TX04_Periph_Driver */
#endif                          /* (__TMPM440_EXB_H) */
