/**
 *******************************************************************************
 * @file    tmpm440_fc.c
 * @brief   This file provides API functions for FC driver.
 * @version V2.0.2.4
 * @date    2014/09/28
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_fc.h"

#if defined(__TMPM440_FC_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup FC 
  * @brief FC driver modules
  * @{
  */

/** @defgroup FC_Private_Defines
  * @{
  */
#ifndef  __BIG_ENDIAN
#ifdef __IAR_SYSTEMS_ICC__
#if 1 != __LITTLE_ENDIAN__
#define     __BIG_ENDIAN
#endif
#endif
#endif

#define BASE_ADDR_LO                (0x00000000UL)
#define BASE_ADDR_HI                (0x00080000UL)

#define SECBIT_SECBIT_PASSWORD      ((uint32_t)0xA74A9D23)
#define SECBIT_SECBIT_SET           ((uint32_t)0x00000001)
#define SECBIT_SECBIT_CLEAR         ((uint32_t)0xFFFFFFFE)

#define SECBIT_SECURITY_MASK        ((uint32_t)0x00000001)
#define SECBIT_DISABLE_FLAG         ((uint32_t)0x00000000)

#define FLCS_BUSY_MASK              ((uint32_t)0x00000001)
#define FLCS_BUSY_FLAG              ((uint32_t)0x00000000)

#define FC_PROTECT_MASK              ((uint32_t)0x00000001)

#define FLCS_BLOCK_FLAG             ((uint32_t)0x00000000)

#define FLASH_AREA                          ((uint32_t)(FLASH_START_ADDR & 0xFFF80000U))

#define FC_CMD_BC1_ADDR                     ((uint32_t) (FLASH_AREA + 0x00005400UL))
#define FC_CMD_BC2_ADDR                     ((uint32_t) (FLASH_AREA + 0x0000AA00UL))

#define FC_CMD_BC1_ADDR_H                   ((uint32_t) (FC_CMD_BC1_ADDR + BASE_ADDR_HI))
#define FC_CMD_BC2_ADDR_H                   ((uint32_t) (FC_CMD_BC2_ADDR + BASE_ADDR_HI))

#define FC_ERASE_CHIP_OVER_TIME             ((uint32_t)0x00CFFFFF)      /* FC_ERASE_CHIP_OVER_TIME > 1.6 */
#define FC_ERASE_BLOCK_OVER_TIME            ((uint32_t)0x000CFFFF)      /* FC_ERASE_BLOCK_OVER_TIME > 0.1s */
#define FC_WRITE_PAGE_OVER_TIME             ((uint32_t)0x000FFFFF)      /* FC_WRITE_PAGE_OVER_TIME > 1.25ms */
#define FC_SET_PROTECT_STATE_OVER_TIME      ((uint32_t)0x000FFFFF)


#define FC_PBA_PART0                0x07U       /*Bit0 ~ Bit2 */
#define FC_PBA_PART1                0x08U       /*Bit3 */
#define FC_PBA_PART2                0x10U       /*Bit4 */
#define FC_PBA_PART0_SHIFT          8U          /*part 0 shift 8 Bits */
#define FC_PBA_PART1_SHIFT          14U         /*part 1 shift 14 Bits */
#define FC_PBA_PART2_SHIFT          15U         /*part 2 shift 15 Bits */


#define FC_PBA_BLOCK_GROUP_0        ((uint32_t *) (FLASH_AREA + 0x00000000U))
#define FC_PBA_BLOCK_GROUP_1        ((uint32_t *) (FLASH_AREA + 1U<<17U))
#define FC_PBA_BLOCK_GROUP_2        ((uint32_t *) (FLASH_AREA + 1U<<19U))
#define FC_PBA_BLOCK_GROUP_3        ((uint32_t *) (FLASH_AREA + 5U<<17U))

#define FC_BLOCK_ADDR_MASK          ((uint32_t)0xFFFFF000)

#define FC_PAGE_SIZE                ((uint32_t)(FLASH_PAGE_SIZE/4U))

typedef struct {
    uint32_t Start_Addr;
    uint32_t End_Addr;
} Block_AddrDef;

const Block_AddrDef block_addr[] = {
    {0U, 32U}, {32U, 64U}, {64U, 96U}, {96U, 128U},
    {128U, 192U}, {192U, 256U},
    {256U, 384U}, {384U, 512U},
    {512U, 768U}, {768U, 1024U}, {1024U, 1280U}, {1280U, 1536U}, {1536U, 1792U}, {1792U, 2048U},
    {2048U, 2080U}, {2080U, 2112U}, {2112U, 2144U}, {2144U, 2176U},
    {2176U, 2240U}, {2240U, 2304U},
    {2304U, 2432U}, {2432U, 2560U},
    {2560U, 2816U}, {2816U, 3072U}, {3072U, 3328U}, {3328U, 3584U}, {3584U, 3840U}, {3840U, 4096U}
};

/** @} */
/* End of group FC_Private_Defines */

/** @defgroup FC_Private_FunctionPrototypes
  * @{
  */
uint8_t FC_AddrToBlockNum(uint32_t Addr);

/** @} */
/* End of group FC_Private_FunctionPrototypes */

/** @defgroup FC_Private_Functions
  * @{
  */
/**
  * @brief  Convert address to block number
  * @param  Addr
  * @retval BlockNum
   */
uint8_t FC_AddrToBlockNum(uint32_t Addr)
{
    uint8_t BlockNum = 0U;
    uint8_t i = 0U;
    uint32_t temp = (Addr - FLASH_START_ADDR) / FLASH_PAGE_SIZE;

    for (i = 0U; i <= FC_BLOCK_MAX; ++i) {
        if ((temp >= block_addr[i].Start_Addr) && (temp < block_addr[i].End_Addr)) {
            BlockNum = i;
            break;
        }
    }
    return BlockNum;
}

/**
  * @brief  Set the value of SECBIT register.
  * @param  NewState: The value of SECBIT register.
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE.
  * @retval None.
  */
void FC_SetSecurityBit(FunctionalState NewState)
{
    uint32_t tmp = TSB_FC->SECBIT;
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* Set FCSECBIT<SECBIT> to "1" that enable security function  */
        tmp |= SECBIT_SECBIT_SET;
    } else {
        /* Set FCSECBIT<SECBIT> to "0" that disable security function  */
        tmp &= SECBIT_SECBIT_CLEAR;
    }
    TSB_FC->SECBIT = SECBIT_SECBIT_PASSWORD;
    TSB_FC->SECBIT = tmp;
}

/**
  * @brief  Get the value of SECBIT register.
  * @param  None.
  * @retval DISABLE or ENABLE.
  */
FunctionalState FC_GetSecurityBit(void)
{
    uint32_t tmp = 0U;
    FunctionalState retval = ENABLE;

    tmp = TSB_FC->SECBIT & SECBIT_SECURITY_MASK;

    /* Check disable flag */
    if (tmp == SECBIT_DISABLE_FLAG) {
        retval = DISABLE;
    } else {                    /* Check enable flag */
        retval = ENABLE;
    }
    return retval;
}

/**
  * @brief  Get the status of the flash auto operation.
  * @param  None.
  * @retval BUSY or DONE.
  */
WorkState FC_GetBusyState(void)
{
    uint32_t tmp = 0U;
    WorkState retval = DONE;

    tmp = TSB_FC->SR & FLCS_BUSY_MASK;

    /* Check busy flag */
    if (tmp == FLCS_BUSY_FLAG) {
        retval = BUSY;
    } else {                    /* Check ready flag */
        retval = DONE;
    }
    return retval;
}

/**
  * @brief  Get the specified block protection state.
  * @param  BlockNum: The flash block number.
  *   This parameter can be one of the following values:
  *   FC_BLOCK_0 to FC_BLOCK_27
  * @retval DISABLE or ENABLE.
  */
FunctionalState FC_GetBlockProtectState(uint8_t BlockNum)
{
    uint32_t tmp = FC_PROTECT_MASK;
    FunctionalState retval = ENABLE;

    /* Check the parameters */
    assert_param(IS_FC_BLOCK_NUM(BlockNum));
    if (BlockNum <= FC_BLOCK_13) {
        tmp = tmp << BlockNum;
        tmp &= TSB_FC->PSRA;
    } else {
        BlockNum -= FC_BLOCK_14;
        tmp = tmp << BlockNum;
        tmp &= TSB_FC->PSRB;
    }
    if (tmp == FLCS_BLOCK_FLAG) {
        /* Return protection status of each block */
        retval = DISABLE;
    } else {
        retval = ENABLE;
    }
    return retval;
}

/**
  * @brief  Program the protection bit to make the specified block protected.
  * @param  BlockNum: The flash block number.
  *   This parameter can be one of the following values:
  *   FC_BLOCK_0 to FC_BLOCK_27
  * @retval FC_SUCCESS, FC_ERROR_PROTECTED, FC_ERROR_OVER_TIME.
  */
FC_Result FC_ProgramBlockProtectState(uint8_t BlockNum)
{
    FC_Result retval = FC_SUCCESS;
    volatile uint32_t *addr1 = (uint32_t *) FC_CMD_BC1_ADDR;
    volatile uint32_t *addr2 = (uint32_t *) FC_CMD_BC2_ADDR;
    volatile uint32_t *PBA = 0U;
    uint32_t counter = FC_SET_PROTECT_STATE_OVER_TIME;
    uint32_t PBA_data;
    uint8_t BlockNumShift = BlockNum;

    /* Check the parameters */
    assert_param(IS_FC_BLOCK_NUM(BlockNum));
    if (BlockNum >= FC_BLOCK_14) {
        BlockNumShift += 2U;
        addr1 = (uint32_t *) FC_CMD_BC1_ADDR_H;
        addr2 = (uint32_t *) FC_CMD_BC2_ADDR_H;
    } else {
        /* Do nothing */
    }
    PBA_data = ((uint32_t) BlockNumShift & FC_PBA_PART0) << FC_PBA_PART0_SHIFT;
    PBA_data |= ((uint32_t) BlockNumShift & FC_PBA_PART1) << FC_PBA_PART1_SHIFT;
    PBA_data |= ((uint32_t) BlockNumShift & FC_PBA_PART2) << FC_PBA_PART2_SHIFT;
    PBA_data += FLASH_AREA;
    if (BlockNum >= FC_BLOCK_14) {
        PBA_data |= BASE_ADDR_HI;
        PBA = (uint32_t *) PBA_data;
    } else {
        PBA = (uint32_t *) PBA_data;
    }

    if (ENABLE == FC_GetBlockProtectState(BlockNum)) {
        retval = FC_ERROR_PROTECTED;
    } else {
#ifdef  __BIG_ENDIAN
        *addr1 = (uint32_t) 0xAA000000; /* bus cycle 1 */
        *addr2 = (uint32_t) 0x55000000; /* bus cycle 2 */
        *addr1 = (uint32_t) 0x9A000000; /* bus cycle 3 */
        *PBA = (uint32_t) 0x9A000000;   /* bus cycle 7 */
#else
        *addr1 = (uint32_t) 0x000000AA; /* bus cycle 1 */
        *addr2 = (uint32_t) 0x00000055; /* bus cycle 2 */
        *addr1 = (uint32_t) 0x0000009A; /* bus cycle 3 */
        *PBA = (uint32_t) 0x0000009A;   /* bus cycle 7 */
#endif
        __DSB();

        while (BUSY == FC_GetBusyState()) {     /* check if FLASH is busy with overtime counter */
            if (!(counter--)) { /* check overtime */
                retval = FC_ERROR_OVER_TIME;
                break;
            } else {
                /* Do nothing */
            }
        }
    }
    return retval;
}

/**
  * @brief  Erase the protection bits to make the specified blocks unprotected.
  * @param  BlockGroup: Block group for erasing of the protection bits.
  *   This parameter can be one of the following values:
  *   FC_BLOCK_GROUP_0: block 0 to 7.
  *   FC_BLOCK_GROUP_1: block 8 to 13.
  *   FC_BLOCK_GROUP_2: block 14 to 21.
  *   FC_BLOCK_GROUP_3: block 22 to 27(for TMPM440F10XBG);
  *                     block 22 to 23(for TMPM440FEXBG);    
  * @retval FC_SUCCESS, FC_ERROR_OVER_TIME.
  */
FC_Result FC_EraseBlockProtectState(uint8_t BlockGroup)
{
    FC_Result retval = FC_SUCCESS;
    volatile uint32_t *addr1 = (uint32_t *) FC_CMD_BC1_ADDR;
    volatile uint32_t *addr2 = (uint32_t *) FC_CMD_BC2_ADDR;
    volatile uint32_t *PBA = FC_PBA_BLOCK_GROUP_0;
    uint32_t counter = FC_SET_PROTECT_STATE_OVER_TIME;

    /* Check the parameters */
    assert_param(IS_FC_BLOCK_GROUP(BlockGroup));

    switch (BlockGroup) {
    case FC_BLOCK_GROUP_0:
        PBA = FC_PBA_BLOCK_GROUP_0;
        break;
    case FC_BLOCK_GROUP_1:
        PBA = FC_PBA_BLOCK_GROUP_1;
        break;
    case FC_BLOCK_GROUP_2:
        PBA = FC_PBA_BLOCK_GROUP_2;
        addr1 = (uint32_t *) FC_CMD_BC1_ADDR_H;
        addr2 = (uint32_t *) FC_CMD_BC2_ADDR_H;
        break;
    case FC_BLOCK_GROUP_3:
        PBA = FC_PBA_BLOCK_GROUP_3;
        addr1 = (uint32_t *) FC_CMD_BC1_ADDR_H;
        addr2 = (uint32_t *) FC_CMD_BC2_ADDR_H;
        break;
    default:
        /* Do nothing */
        break;
    }
#ifdef  __BIG_ENDIAN
    *addr1 = (uint32_t) 0xAA000000;     /* bus cycle 1 */
    *addr2 = (uint32_t) 0x55000000;     /* bus cycle 2 */
    *addr1 = (uint32_t) 0x6A000000;     /* bus cycle 3 */
    *PBA = (uint32_t) 0x6A000000;       /* bus cycle 7 */
#else
    *addr1 = (uint32_t) 0x000000AA;     /* bus cycle 1 */
    *addr2 = (uint32_t) 0x00000055;     /* bus cycle 2 */
    *addr1 = (uint32_t) 0x0000006A;     /* bus cycle 3 */
    *PBA = (uint32_t) 0x0000006A;       /* bus cycle 7 */
#endif
    __DSB();

    while (BUSY == FC_GetBusyState()) { /* check if FLASH is busy with overtime counter */
        if (!(counter--)) {     /* check overtime */
            retval = FC_ERROR_OVER_TIME;
            break;
        } else {
            /* Do nothing */
        }
    }

    return retval;
}

/**
  * @brief  Write data to the specified page.
  * @param  PageAddr: The page start address.
  *         Data: The pointer to data buffer to be written into the page.
  *         The data size should be FC_PAGE_SIZE.
  * @retval FC_SUCCESS, FC_ERROR_PROTECTED, FC_ERROR_OVER_TIME.
  */
FC_Result FC_WritePage(uint32_t PageAddr, uint32_t * Data)
{
    FC_Result retval = FC_SUCCESS;
    volatile uint32_t *addr1 = (uint32_t *) FC_CMD_BC1_ADDR;
    volatile uint32_t *addr2 = (uint32_t *) FC_CMD_BC2_ADDR;
    volatile uint32_t *addr3 = (uint32_t *) PageAddr;
    uint32_t counter = FC_WRITE_PAGE_OVER_TIME;
    uint32_t i = 0U;
    uint32_t *source = Data;
    uint8_t BlockNum = FC_BLOCK_0;

    assert_param(IS_FC_ADDR(PageAddr)); /* Check whether it is in the flash address range */
    assert_param(IS_FC_PAGE_ADDR(PageAddr));    /* Check whether it is a page start address */
    assert_param(IS_POINTER_NOT_NULL(Data));

    BlockNum = FC_AddrToBlockNum(PageAddr);

    if (BlockNum >= FC_BLOCK_14) {
        addr1 = (uint32_t *) FC_CMD_BC1_ADDR_H;
        addr2 = (uint32_t *) FC_CMD_BC2_ADDR_H;
    } else {
        /* Do nothing */
    }

    if (ENABLE == FC_GetBlockProtectState(BlockNum)) {
        retval = FC_ERROR_PROTECTED;
    } else {
#ifdef  __BIG_ENDIAN
        *addr1 = (uint32_t) 0xAA000000;
        *addr2 = (uint32_t) 0x55000000;
        *addr1 = (uint32_t) 0xA0000000;
#else

        *addr1 = (uint32_t) 0x000000AA; /* bus cycle 1 */
        *addr2 = (uint32_t) 0x00000055; /* bus cycle 2 */
        *addr1 = (uint32_t) 0x000000A0; /* bus cycle 3 */
#endif
        for (i = 0U; i < FC_PAGE_SIZE; i++) {
            *addr3 = *source;
            source++;
        }
        __DSB();

        while (BUSY == FC_GetBusyState()) {     /* check if FLASH is busy with overtime counter */
            if (!(counter--)) { /* check overtime */
                retval = FC_ERROR_OVER_TIME;
                break;
            } else {
                /* Do nothing */
            }
        }
    }

    return retval;
}

/**
  * @brief  Erase the contents of the specified block.
  * @param  BlockAddr: The block start address.
  * @retval FC_SUCCESS, FC_ERROR_PROTECTED, FC_ERROR_OVER_TIME.
  */
FC_Result FC_EraseBlock(uint32_t BlockAddr)
{
    FC_Result retval = FC_SUCCESS;
    volatile uint32_t *addr1 = (uint32_t *) FC_CMD_BC1_ADDR;
    volatile uint32_t *addr2 = (uint32_t *) FC_CMD_BC2_ADDR;
    volatile uint32_t *BA = 0U;
    uint32_t counter = FC_ERASE_BLOCK_OVER_TIME;
    uint8_t BlockNum = FC_BLOCK_4;

    assert_param(IS_FC_ADDR(BlockAddr));        /* Check whether it is in the flash address range */

    BlockNum = FC_AddrToBlockNum(BlockAddr);
    if (BlockNum >= FC_BLOCK_14) {
        addr1 = (uint32_t *) FC_CMD_BC1_ADDR_H;
        addr2 = (uint32_t *) FC_CMD_BC2_ADDR_H;
    } else {
        /* Do nothing */
    }
    BlockAddr &= FC_BLOCK_ADDR_MASK;
    BA = (uint32_t *) BlockAddr;

    if (ENABLE == FC_GetBlockProtectState(BlockNum)) {
        retval = FC_ERROR_PROTECTED;
    } else {
#ifdef  __BIG_ENDIAN
        *addr1 = (uint32_t) 0xAA000000; /* bus cycle 1 */
        *addr2 = (uint32_t) 0x55000000; /* bus cycle 2 */
        *addr1 = (uint32_t) 0x80000000; /* bus cycle 3 */
        *addr1 = (uint32_t) 0xAA000000; /* bus cycle 4 */
        *addr2 = (uint32_t) 0x55000000; /* bus cycle 5 */
        *BA = (uint32_t) 0x30000000;    /* bus cycle 6 */
#else
        *addr1 = (uint32_t) 0x000000AA; /* bus cycle 1 */
        *addr2 = (uint32_t) 0x00000055; /* bus cycle 2 */
        *addr1 = (uint32_t) 0x00000080; /* bus cycle 3 */
        *addr1 = (uint32_t) 0x000000AA; /* bus cycle 4 */
        *addr2 = (uint32_t) 0x00000055; /* bus cycle 5 */
        *BA = (uint32_t) 0x00000030;    /* bus cycle 6 */
#endif
        __DSB();

        while (BUSY == FC_GetBusyState()) {     /* check if FLASH is busy with overtime counter */
            if (!(counter--)) { /* check overtime */
                retval = FC_ERROR_OVER_TIME;
                break;
            } else {
                /* Do nothing */
            }
        }
    }

    return retval;
}

/**
  * @brief  Erase the contents of the entire chip.
  * @param  None.
  * @retval FC_SUCCESS, FC_ERROR_PROTECTED, FC_ERROR_OVER_TIME.
  */
FC_Result FC_EraseChip(void)
{
    FC_Result retval = FC_SUCCESS;
    volatile uint32_t *addr1 = (uint32_t *) FC_CMD_BC1_ADDR;
    volatile uint32_t *addr2 = (uint32_t *) FC_CMD_BC2_ADDR;
    uint32_t counter = FC_ERASE_CHIP_OVER_TIME;
    FunctionalState result;
    uint8_t BlockNum = FC_BLOCK_0;
    for (BlockNum = FC_BLOCK_0; BlockNum <= FC_BLOCK_MAX; ++BlockNum) {
        result = FC_GetBlockProtectState(BlockNum);
        if (result == ENABLE) {
            break;
        } else {
            /* Do nothing */
        }
    }

    if (ENABLE == result) {
        retval = FC_ERROR_PROTECTED;
    } else {
#ifdef  __BIG_ENDIAN
        *addr1 = (uint32_t) 0xAA000000; /* bus cycle 1 */
        *addr2 = (uint32_t) 0x55000000; /* bus cycle 2 */
        *addr1 = (uint32_t) 0x80000000; /* bus cycle 3 */
        *addr1 = (uint32_t) 0xAA000000; /* bus cycle 4 */
        *addr2 = (uint32_t) 0x55000000; /* bus cycle 5 */
        *addr1 = (uint32_t) 0x10000000; /* bus cycle 6 */
#else
        *addr1 = (uint32_t) 0x000000AA; /* bus cycle 1 */
        *addr2 = (uint32_t) 0x00000055; /* bus cycle 2 */
        *addr1 = (uint32_t) 0x00000080; /* bus cycle 3 */
        *addr1 = (uint32_t) 0x000000AA; /* bus cycle 4 */
        *addr2 = (uint32_t) 0x00000055; /* bus cycle 5 */
        *addr1 = (uint32_t) 0x00000010; /* bus cycle 6 */
#endif
        __DSB();

        while (BUSY == FC_GetBusyState()) {     /* check if FLASH is busy with overtime counter */
            if (!(counter--)) { /* check overtime */
                retval = FC_ERROR_OVER_TIME;
                break;
            } else {
                /* Do nothing */
            }
        }
    }

    return retval;
}

/**
  * @brief  Enable or Disable flash buffer
  * @param  NewState: Flash buffer is enabled/disabled.
  *   This parameter can be one of the following values:
  *   DISABLE or ENABLE.
  * @retval None
  */
void FC_SetCtrlReg(FunctionalState NewState)
{
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    TSB_FC->CR = (uint32_t) NewState;
}

/** @} */
/* End of group FC_Exported_Functions */

/** @} */
/* End of group FC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /*(__TMPM440_FC_H) */
