/**
 *******************************************************************************
 * @file    tmpm440_kscan.c
 * @brief   This file provides API functions for Key Scan driver.
 * @version V2.0.2.3
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_kscan.h"

#if defined (__TMPM440_KSCAN_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup KSCAN 
  * @brief KSCAN driver modules
  * @{
  */
/** @defgroup KSCAN_Private_Defines
  * @{
  */
#define     KSCAN_SWRST_CMD1    ((uint32_t)0x00000002)
#define     KSCAN_SWRST_CMD2    ((uint32_t)0x00000001)


/** @} */
/* End of group KSCAN_Private_Defines */



/** @defgroup KSCAN_Exported_Functions
  * @{
  */

/**
  * @brief  Enable KSCAN function.
  * @param  None
  * @retval None
  */
void KSCAN_Enable(void)
{
    TSB_KS_EN_EN = ENABLE;
}

/**
  * @brief  Disable KSCAN function.
  * @param  None
  * @retval None
  */

void KSCAN_Disable(void)
{
    TSB_KS_EN_EN = DISABLE;
}

/**
  * @brief  Select KSCAN operation clock (ksclk).
  * @param  ksclk: KSCAN operation clock.
  *   This parameter can be KSCAN_KSCL_FS or KSCAN_KSCL_TBOUT.
  * @retval None
  */
void KSCAN_SetSCLK(uint8_t ksclk)
{
    assert_param(IS_KSCAN_KSCLK(ksclk));

    TSB_KS_EN_SC = ksclk;
}

/**
  * @brief  Set KSCAN input control.
  * @param  InputCH: Select which input channel to set.
  *   This parameter can be:
  *   KSCAN_CH_0,KSCAN_CH_1,KSCAN_CH_2,KSCAN_CH_3,KSCAN_CH_4,KSCAN_CH_5,
  *   KSCAN_CH_6,KSCAN_CH_7,KSCAN_CH_ALL or combination of channels.
  * @param  NewState: New state of mask keyscan input.
  *   This parameter can be ENABLE(Masked) or DISABLE(Not masked).  
  * @retval None
  */
void KSCAN_SetInputCtrlMask(uint8_t InputCH, FunctionalState NewState)
{
    uint8_t tmp = (uint8_t) TSB_KS->ICR;
    assert_param(IS_KSCAN_INOUTCH(InputCH));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        tmp |= InputCH;
    } else {
        tmp &= (uint8_t) (~InputCH);
    }

    TSB_KS->ICR = tmp;
}

/**
  * @brief  Set KSCAN output control.
  * @param  OutputCH: Select which output channel to set.
  *   This parameter can:
  *   KSCAN_CH_0,KSCAN_CH_1,KSCAN_CH_2,KSCAN_CH_3,KSCAN_CH_4,KSCAN_CH_5,
  *   KSCAN_CH_6,KSCAN_CH_7,KSCAN_CH_ALL or combination of channels.
  * @param  NewState: New state of keyscan output.
  *   This parameter can be ENABLE(Output) or DISABLE(Not Output).  
  * @retval None
  */

void KSCAN_SetOutputCtrl(uint8_t OutputCH, FunctionalState NewState)
{
    uint32_t tmp = TSB_KS->OCR;
    uint32_t kslo = OutputCH;
    assert_param(IS_KSCAN_INOUTCH(OutputCH));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    kslo = kslo << 8U;
    if (NewState == ENABLE) {
        tmp |= kslo;
    } else {
        tmp &= (~kslo);
    }

    TSB_KS->OCR = tmp;
}

/**
  * @brief  Set the strobe output frame time of KSCAN outputs.
  * @param  cycle: set strobe width cycle.
  *   This parameter can be 1 to 255.
  * @retval None
  */
void KSCAN_SetStrobeOutput(uint8_t cycle)
{
    uint32_t tmp = TSB_KS->OCR;
    assert_param(IS_KSCAN_STROBECYCLE(cycle));
    tmp &= 0xFF00U;
    tmp |= cycle;
    TSB_KS->OCR = tmp;
}

/**
  * @brief  start key strobe outputs and counters
  * @param  None
  * @retval None
  */
void KSCAN_Start(void)
{
    TSB_KS_CR_START = ENABLE;
}

/**
  * @brief  Stop key strobe outputs and counters
  * @param  None
  * @retval None
  */
void KSCAN_Stop(void)
{
    TSB_KS_CR_START = DISABLE;
}

/**
  * @brief  Read the kscan start condition
  * @param  None
  * @retval start condition
  */
uint8_t KSCAN_ReadStart(void)
{
    return ((uint8_t) TSB_KS_CR_START);
}

/**
  * @brief  Initializes KSBR to 0
  * @param  None
  * @retval None
  */
void KSCAN_KSBRInitial(void)
{
    TSB_KS_CR_BRRST = 0x01U;

}

/**
  * @brief  KSCAN software reset
  * @param  None
  * @retval None
  */
void KSCAN_SWReset(void)
{
    TSB_KS->CR = KSCAN_SWRST_CMD1;
    TSB_KS->CR = KSCAN_SWRST_CMD2;
}

/**
  * @brief  Set consecutive matches number for key on/off determination.
  * @param  match: the consecutive match number.
  *   This parameter can be KSCAN_MATCH_2C,KSCAN_MATCH_4C.    
  * @retval None
  */
void KSCAN_SetConsecutiveMatches(uint8_t match)
{
    assert_param(IS_KSCAN_MATCH(match));
    if (match == KSCAN_MATCH_4C) {
        TSB_KS->CR |= (1U << 4U);
    } else {
        TSB_KS->CR &= 0x0FU;
    }
}

/**
  * @brief  Set a chattering cancel time:16/ksclk * timeN
  * @param  timeN: a chattering cancel time index.
  *   This parameter can be 0 to 255.
  * @retval None
  */
void KSCAN_SetChatCancelTime(uint8_t timeN)
{
    uint32_t tmp;
    tmp = TSB_KS->CTR;
    tmp &= 0x700U;
    tmp |= timeN;
    TSB_KS->CTR = tmp;
}

/**
  * @brief  Set a strobe width.
  * @param  StrobeWidth: strobe width data.
  *   This parameter can be 0 to 5.
  * @retval None
  */
void KSCAN_SetStrobeWidth(uint8_t StrobeWidth)
{
    uint32_t reg = 0U;

    assert_param(IS_KSCAN_STROBEWIDTH(StrobeWidth));
    reg = TSB_KS->CTR;
    reg &= (uint32_t) 0x000000FF;
    reg |= ((uint32_t) StrobeWidth) << 8U;
    TSB_KS->CTR = reg;
}

/**
  * @brief  A read result of KSCAN output can be checked.
  * @param  output: Read this output.
  *   This parameter can be 0 to 7.
  * @retval 8 bit data
  */
uint8_t KSCAN_ReadBufChecked(uint8_t output)
{
    uint8_t retval = 0U;

    assert_param(IS_KSCAN_CH(output));
    if (output < 4U) {
        output = output << 3U;
        retval = (uint8_t) (TSB_KS->BR0 >> output);
    } else {
        output -= 4U;
        output = output << 3U;
        retval = (uint8_t) (TSB_KS->BR1 >> output);
    }
    return (retval);
}


/**
  * @brief  Masks read buffer of corresponding KSCAN output.
  * @param  MaskCH: Set this output to mask read buffer.
  *   This parameter can be 0 to 7.  
  * @param  MaskBit: Set the bit of output to masked or not masked.
  *   This parameter can be 1 to 255.
  * @param  NewState: New state of KSCAN read buffer.
  *   This parameter can be ENABLE(Masked) or DISABLE(Not masked).  
  * @retval None
  */
void KSCAN_MaskReadBuf(uint32_t MaskCH, uint32_t MaskBit, FunctionalState NewState)
{

    uint32_t volatile *BMR;

    /* Check the parameters */
    assert_param(IS_KSCAN_CH(MaskCH));
    assert_param(IS_KSCAN_MASKBIT(MaskBit));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (MaskCH < 4U) {
        BMR = &TSB_KS->BMR0;
    } else {
        BMR = &TSB_KS->BMR1;
        MaskCH = MaskCH - 4U;
    }
    MaskCH = MaskCH << 3U;
    if (NewState == ENABLE) {
        *BMR |= (uint32_t) (MaskBit << MaskCH);
    } else {
        MaskBit = MaskBit << MaskCH;
        MaskBit = ~MaskBit;
        *BMR &= MaskBit;
    }
}

/**
  * @brief  Specifies a KSCAN interrupt request.
  * @param  NewState: New state of KSCAN  Interrupt request.
  *   This parameter can be ENABLE(Interrupts are not masked)
  *   or DISABLE(Interrupts are masked).
  * @retval None
  */
void KSCAN_SetINTReq(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    TSB_KS_INTCR_INTEN = NewState;
}


/** @} */
/* End of group PHC_Exported_Functions */

/** @} */
/* End of group PHC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_PHC_H) */
