/**
 *******************************************************************************
 * @file    tmpm440_tmrd.c
 * @brief   This file provides API functions for TMRD driver.
 * @version V2.0.2.3
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_tmrd.h"

#if defined(__TMPM440_TMRD_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup TMRD
  * @brief TMRD driver modules
  * @{
  */

/** @defgroup TMRD_Private_Defines
  * @{
  */

#define EN_TDEN1_SET          (0x00000080UL)
#define EN_TDEN1_CLEAR        (0xFFFFFF7FUL)
#define EN_TDEN0_SET          (0x00000040UL)
#define EN_TDEN0_CLEAR        (0xFFFFFFBFUL)
#define EN_TDHALT_SET         (0x00000020UL)
#define EN_TDHALT_CLEAR       (0xFFFFFFDFUL)
#define CONF_TDI2TD1_SET      (0x00000080UL)
#define CONF_TDI2TD1_CLEAR    (0xFFFFFF7FUL)
#define CONF_TDI2TD0_SET      (0x00000040UL)
#define CONF_TDI2TD0_CLEAR    (0xFFFFFFBFUL)
#define CONF_TMRDMOD_MASK     (0xFFFFFFF8UL)
#define MOD_TDIV1_SET         (0x00000080UL)
#define MOD_TDIV1_CLEAR       (0xFFFFFF7FUL)
#define MOD_TDIV0_SET         (0x00000040UL)
#define MOD_TDIV0_CLEAR       (0xFFFFFFBFUL)
#define MOD_TDCLK_MASK        (0xFFFFFFF0UL)
#define CR_TDISO_MASK         (0xFFFFFFFCUL)
#define CR_TDMDCY_0_MASK      (0xFFFFFF1FUL)
#define CR_TDMDCY_1_MASK      (0xFFFFF1FFUL)
#define CR_TDMDPT_0_MASK      (0xFFFFFFEFUL)
#define CR_TDMDPT_1_MASK      (0xFFFFFEFFUL)
#define RG24_TIMING_MASK      (0x0000000FUL)
#define BCR_PHSCHG_SET        (0x00000010UL)
#define BCR_PHSCHG_CLEAR      (0x00000000UL)

/** @} */
/* End of group TMRD_Private_Defines */

/** @defgroup TMRD_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group TMRD_Private_FunctionPrototypes */

/** @defgroup TMRD_Private_Functions
  * @{
  */

/** @} */
/* End of group TMRD_Private_Functions */

/** @defgroup TMRD_Exported_Functions
  * @{
  */

/**
  * @brief  Enable clock signal input to TMRD.
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @retval None
  */
void TMRD_Enable(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));

    /* Set TDxEN<TDEN0, TDEN1> */
    TDx->EN |= (CHx == TMRD_UNIT_CH_0) ? (EN_TDEN0_SET) : (EN_TDEN1_SET);
}

/**
  * @brief  Disable clock signal input to TMRD.
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @retval None
  */
void TMRD_Disable(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));

    /* Clear TD0EN<TDEN0, TDEN1> */
    TDx->EN &= (CHx == TMRD_UNIT_CH_0) ? (EN_TDEN0_CLEAR) : (EN_TDEN1_CLEAR);
}

/**
  * @brief  Set TMRD operation if a HALT instruction is 
  *   executed during debugging.
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  RunState: TMRD operation status setting.
  *   This parameter can be one of the following values:
  *   TMRD_RUN, TMRD_STOP.
  * @retval None
  */
void TMRD_SetRunStateInHalt(TSB_TD_TypeDef * TDx, uint8_t RunState)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_RUN_STATE(RunState));

    /* Set TDxEN<TDHALT> */
    TDx->EN = (RunState == TMRD_RUN) ? (TDx->EN | EN_TDHALT_SET) : (TDx->EN & EN_TDHALT_CLEAR);
}

/**
  * @brief  Set TMRD operation during the IDLE mode. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  RunState: TMRD operation status setting.
  *   This parameter can be one of the following values:
  *   TMRD_RUN, TMRD_STOP.
  * @retval None
  */
void TMRD_SetRunStateInIdle(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx, uint8_t RunState)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_RUN_STATE(RunState));

    /* Set TDxCONF<TDI2TD0, TDI2TD1> */
    if (RunState == TMRD_RUN) {
        TDx->CONF |= (CHx == TMRD_UNIT_CH_0) ? (CONF_TDI2TD0_SET) : (CONF_TDI2TD1_SET);
    } else {
        TDx->CONF &= (CHx == TMRD_UNIT_CH_0) ? (CONF_TDI2TD0_CLEAR) : (CONF_TDI2TD1_CLEAR);
    }
}

/**
  * @brief  Set the operation mode for TMRD. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  Mode: Operation mode of TMRD.
  *   This parameter can be one of the following values:
  *   TMRD_MODE_BOTH_TMR, TMRD_MODE_0TMR_1PPG, 
  *   TMRD_MODE_0PPG_1TMR, TMRD_MODE_BOTH_PPG, 
  *   TMRD_MODE_INTERLOCK_TMR, TMRD_MODE_INTERLOCK_PPG_2CH,
  *   TMRD_MODE_INTERLOCK_PPG_3CH.
  * @retval None
  */
void TMRD_SetMode(TSB_TD_TypeDef * TDx, uint8_t Mode)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_MODE(Mode));

    /* Set TDxCONF<TMRDMOD> */
    tmp = TDx->CONF & CONF_TMRDMOD_MASK;
    tmp |= Mode;
    TDx->CONF = tmp;
}

/**
  * @brief  Set the clock prescaler of TMRD. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  ClkDiv: Clock prescaler factor.
  *   This parameter can be one of the following values:
  *   TMRD_CLK_DIV_1, TMRD_CLK_DIV_2, TMRD_CLK_DIV_4, 
  *   TMRD_CLK_DIV_8, TMRD_CLK_DIV_16.
  * @retval None
  */
void TMRD_SetClkDivision(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx, uint8_t ClkDiv)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_CLK_DIV(ClkDiv));

    if (CHx == TMRD_UNIT_CH_0) {
        /* Set TDxMOD<TDCLK> */
        tmp = TSB_TD0->MOD & MOD_TDCLK_MASK;
        tmp |= ClkDiv;
        TSB_TD0->MOD = tmp;
    } else {
        /* Set TDxMOD<TDCLK> */
        tmp = TSB_TD1->MOD & MOD_TDCLK_MASK;
        tmp |= ClkDiv;
        TSB_TD1->MOD = tmp;
    }
}

/**
  * @brief  Select the timer upcounter operation when match the cycle. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  UpCntCtrl: Up counter clear operation mode.
  *   This parameter can be one of the following values:
  *   TMRD_FREE_RUN, TMRD_AUTO_CLEAR. 
  * @retval None
  */
void TMRD_SetUpCntCtrl(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx, uint8_t UpCntCtrl)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_UC_CTRL(UpCntCtrl));

    /* Set TDxMOD<TDCLE> */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0_MOD_TDCLE = (UpCntCtrl == TMRD_AUTO_CLEAR) ? 1UL : 0UL;
    } else {
        TSB_TD1_MOD_TDCLE = (UpCntCtrl == TMRD_AUTO_CLEAR) ? 1UL : 0UL;
    }
}

/**
  * @brief  Set the initial leading/trailing edge of PPG channels.
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  PPGChannel: PPG output channel.
  *   This parameter can be one of the following values:
  *   TMRD_PPG_CHANNEL_A0, TMRD_PPG_CHANNEL_A1, 
  *   TMRD_PPG_CHANNEL_B0, TMRD_PPG_CHANNEL_B1.
  * @param  WaveEdge: Wave edge of signal.
  *   This parameter can be one of the following values:
  *   TMRD_WAVE_EDGE_RISING, TMRD_WAVE_EDGE_FALLING. 
  * @retval None
  */
void TMRD_SetPPGInitLeadingEdge(TSB_TD_TypeDef * TDx, uint8_t PPGChannel, uint8_t WaveEdge)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_PPG_CHANNEL(PPGChannel));
    assert_param(IS_TMRD_WAVE_EDGE(WaveEdge));

    /* Set TDxMOD<TDIV0, TDIV1> */
    switch (PPGChannel) {
    case TMRD_PPG_CHANNEL_A0:
        TSB_TD0->MOD = (WaveEdge == TMRD_WAVE_EDGE_FALLING) ? (TSB_TD0->MOD | MOD_TDIV0_SET) :
            (TSB_TD0->MOD & MOD_TDIV0_CLEAR);
        break;
    case TMRD_PPG_CHANNEL_A1:
        TSB_TD0->MOD = (WaveEdge == TMRD_WAVE_EDGE_FALLING) ? (TSB_TD0->MOD | MOD_TDIV1_SET) :
            (TSB_TD0->MOD & MOD_TDIV1_CLEAR);
        break;
    case TMRD_PPG_CHANNEL_B0:
        TSB_TD1->MOD = (WaveEdge == TMRD_WAVE_EDGE_FALLING) ? (TSB_TD1->MOD | MOD_TDIV0_SET) :
            (TSB_TD1->MOD & MOD_TDIV0_CLEAR);
        break;
    case TMRD_PPG_CHANNEL_B1:
        TSB_TD1->MOD = (WaveEdge == TMRD_WAVE_EDGE_FALLING) ? (TSB_TD1->MOD | MOD_TDIV1_SET) :
            (TSB_TD1->MOD & MOD_TDIV1_CLEAR);
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set a write path to the compare register. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  WritePath: The path to update compare register.
  *   This parameter can be one of the following values:
  *   TMRD_CMP_WRITE_DIRECT, TMRD_CMP_WRITE_INDIRECT. 
  * @retval None
  */
void TMRD_SetCMPRegWritePath(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx, uint8_t WritePath)
{

    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_CMP_WRITE_PATH(WritePath));

    /* Set TDxCR<TDRDE> */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0_CR_TDRDE = (WritePath == TMRD_CMP_WRITE_DIRECT) ? 0UL : 1UL;
    } else {
        TSB_TD1_CR_TDRDE = (WritePath == TMRD_CMP_WRITE_DIRECT) ? 0UL : 1UL;
    }

}

/**
  * @brief  Set the interrupt source of compare interrupt 0. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  INTSrc: Select the interrupt factor.
  *   This parameter can be one of the following values:
  *   TMRD_INT_NONE, TMRD_INT_MATCH_CYCLE, 
  *   TMRD_INT_MATCH_PHASE (only TMRD0 has), TMRD_INT_UC_OVERFLOW. 
  * @retval None
  */
void TMRD_SetCMP0INTSrc(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx, uint8_t INTSrc)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));

    if (CHx == TMRD_UNIT_CH_0) {
        assert_param(IS_TMRD_TD0_INT_SRC(INTSrc));
    } else {
        assert_param(IS_TMRD_TD1_INT_SRC(INTSrc));
    }
    /* Set TDxCR<TDISO> */
    if (CHx == TMRD_UNIT_CH_0) {
        tmp = TSB_TD0->CR & CR_TDISO_MASK;
        tmp |= INTSrc;
        TSB_TD0->CR = tmp;
    } else {
        tmp = TSB_TD1->CR & CR_TDISO_MASK;
        tmp |= INTSrc;
        TSB_TD1->CR = tmp;
    }
}

/**
  * @brief  Set the 1bit modulation cycle. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  PPGChannel: PPG output channel.
  *   This parameter can be one of the following values:
  *   TMRD_PPG_CHANNEL_A0, TMRD_PPG_CHANNEL_A1, 
  *   TMRD_PPG_CHANNEL_B0, TMRD_PPG_CHANNEL_B1.
  * @param  BitModCycle: Select the 1bit modulation cycle.
  *   This parameter can be one of the following values:
  *   TMRD_1BITMOD_CYCLE_NONE, TMRD_1BITMOD_CYCLE_2TIMES, TMRD_1BITMOD_CYCLE_4TIMES, 
  *   TMRD_1BITMOD_CYCLE_8TIMES or TMRD_1BITMOD_CYCLE_16TIMES. 
  * @retval None
  */
void TMRD_SetBitModulationCycle(TSB_TD_TypeDef * TDx, uint8_t PPGChannel, uint8_t BitModCycle)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_PPG_CHANNEL(PPGChannel));
    assert_param(IS_TMRD_1BITMOD_CYCLE(BitModCycle));

    /* Set TDmnCR */
    switch (PPGChannel) {
    case TMRD_PPG_CHANNEL_A0:
        tmp = TSB_TD0->CR & CR_TDMDCY_0_MASK;
        tmp |= ((uint32_t) BitModCycle << 5U);
        TSB_TD0->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_A1:
        tmp = TSB_TD0->CR & CR_TDMDCY_1_MASK;
        tmp |= ((uint32_t) BitModCycle << 9U);
        TSB_TD0->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_B0:
        tmp = TSB_TD1->CR & CR_TDMDCY_0_MASK;
        tmp |= ((uint32_t) BitModCycle << 5U);
        TSB_TD1->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_B1:
        tmp = TSB_TD1->CR & CR_TDMDCY_1_MASK;
        tmp |= ((uint32_t) BitModCycle << 9U);
        TSB_TD1->CR = tmp;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set the 1bit modulation update timing. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  PPGChannel: PPG output channel.
  *   This parameter can be one of the following values:
  *   TMRD_PPG_CHANNEL_A0, TMRD_PPG_CHANNEL_A1, 
  *   TMRD_PPG_CHANNEL_B0, TMRD_PPG_CHANNEL_B1.
  * @param  NewState: The state of the 1bit modulation .
  *   This parameter can be one of the following values:
  *   DISABLE(1bit modulation), ENABLE(CPxx).
  * @retval None
  */
void TMRD_SetBitModUpdateTiming(TSB_TD_TypeDef * TDx, uint8_t PPGChannel, FunctionalState NewState)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_PPG_CHANNEL(PPGChannel));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* Set TDmnCR */
    switch (PPGChannel) {
    case TMRD_PPG_CHANNEL_A0:
        tmp = TSB_TD0->CR & CR_TDMDPT_0_MASK;
        tmp |= ((uint32_t) NewState << 4U);
        TSB_TD0->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_A1:
        tmp = TSB_TD0->CR & CR_TDMDPT_1_MASK;
        tmp |= ((uint32_t) NewState << 8U);
        TSB_TD0->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_B0:
        tmp = TSB_TD1->CR & CR_TDMDPT_0_MASK;
        tmp |= ((uint32_t) NewState << 4U);
        TSB_TD1->CR = tmp;
        break;
    case TMRD_PPG_CHANNEL_B1:
        tmp = TSB_TD1->CR & CR_TDMDPT_1_MASK;
        tmp |= ((uint32_t) NewState << 8U);
        TSB_TD1->CR = tmp;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set the count operation of TMRD. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  RunState: The counter operation of TMRD.
  *   This parameter can be one of the following values:
  *   TMRD_RUN, TMRD_STOP. 
  * @retval None
  */
void TMRD_SetRunState(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx, uint8_t RunState)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_TMRD_RUN_STATE(RunState));

    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0_RUN_TDRUN = (RunState == TMRD_RUN) ? 1UL : 0UL;
    } else {
        TSB_TD1_RUN_TDRUN = (RunState == TMRD_RUN) ? 1UL : 0UL;
    }
}

/**
  * @brief  Set the phase relation of phase B to phase A. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  PhaseRelation: The phase relation of phase B to the phase A.
  *   This parameter can be one of the following values:
  *   TMRD_PHASE_DELAY_OR_SAME, TMRD_PHASE_FAST_OR_SAME.
  * @retval None
  */
void TMRD_SetPhaseRelation(TSB_TD_TypeDef * TDx, uint8_t PhaseRelation)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_PHASE_RELATION(PhaseRelation));

    /* Set TDBCR<PHSCHG> */
    TDx->BCR = (PhaseRelation == TMRD_PHASE_FAST_OR_SAME) ? (BCR_PHSCHG_SET) : (BCR_PHSCHG_CLEAR);
}

/**
  * @brief  Enable to update the compare register. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD. 
  * @param  UpdateCHx: Select shift channel.
  *   This parameter can be one of the following values:
  *   TMRD_UPDATE_CH_00, TMRD_UPDATE_CH_01, 
  *   TMRD_UPDATE_CH_10, TMRD_UPDATE_CH_11,
  *   or combination of the channels above.
  * @retval None
  */
void TMRD_EnableUpdateCMPReg(TSB_TD_TypeDef * TDx, uint8_t UpdateCHx)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_UPDATE_CH(UpdateCHx));

    TDx->BCR |= UpdateCHx;
}

/**
  * @brief  Enable or disable the DMA request of INTTDxCMP0 signal. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  NewState: The state of DMA request.
  *   This parameter can be one of the following values:
  *   ENABLE, DISABLE. 
  * @retval None
  */
void TMRD_SetDMAReq(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* Set TDxDMA<DMAEN> */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0_DMA_DMAEN = (NewState == ENABLE) ? 1UL : 0UL;
    } else {
        TSB_TD1_DMA_DMAEN = (NewState == ENABLE) ? 1UL : 0UL;
    }
}

/**
  * @brief  Initialize TMRD timing parameters. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  CHx: Select the TMRD channel of unit.
  *   This parameter can be one of the following values:
  *   TMRD_UNIT_CH_0, TMRD_UNIT_CH_1.
  * @param  TimingStruct: The pointer of TMRD timing 
  *   parameters structure.
  * @retval None
  */
void TMRD_SetInitTiming(TSB_TD_TypeDef * TDx, TMRD_UNIT_Channel CHx,
                        TMRD_TimingTypeDef * TimingStruct)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_ALL_CHANNEL(CHx));
    assert_param(IS_POINTER_NOT_NULL(TimingStruct));
    assert_param(IS_TMRD_TIMING_RANGE(TimingStruct->Cycle));

    /* Set TDxRGm */
    if (CHx == TMRD_UNIT_CH_0) {
        TSB_TD0->RG0 = TimingStruct->Cycle - 1U;
        TSB_TD0->RG1 = TimingStruct->LeadingTiming0;
        TSB_TD0->RG2 = ((uint32_t) TimingStruct->TrailingTiming0 << 4U) |
            ((uint32_t) TimingStruct->BitModulationRate0 & 0x0FU);
        TSB_TD0->RG3 = TimingStruct->LeadingTiming1;
        TSB_TD0->RG4 = ((uint32_t) TimingStruct->TrailingTiming1 << 4U) |
            ((uint32_t) TimingStruct->BitModulationRate1 & 0x0FU);
        TSB_TD0->RG5 = TimingStruct->PhaseShiftTiming;
    } else {
        TSB_TD1->RG0 = TimingStruct->Cycle - 1U;
        TSB_TD1->RG1 = TimingStruct->LeadingTiming0;
        TSB_TD1->RG2 = ((uint32_t) TimingStruct->TrailingTiming0 << 4U) |
            ((uint32_t) TimingStruct->BitModulationRate0 & 0x0FU);
        TSB_TD1->RG3 = TimingStruct->LeadingTiming1;
        TSB_TD1->RG4 = ((uint32_t) TimingStruct->TrailingTiming1 << 4U) |
            ((uint32_t) TimingStruct->BitModulationRate1 & 0x0FU);
    }

}

/**
  * @brief  Change the specified TMRD timing value. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  TimingType: Timing parameters type.
  *   This parameter can be one of the following values:
  *   TMRD_TIMING_TD0_CYCLE, TMRD_TIMING_A0_LEADING, 
  *   TMRD_TIMING_A0_TRAILING, TMRD_TIMING_A1_LEADING, 
  *   TMRD_TIMING_A1_TRAILING, TMRD_TIMING_PHASE_SHIFT, 
  *   TMRD_TIMING_TD1_CYCLE, TMRD_TIMING_B0_LEADING, 
  *   TMRD_TIMING_B0_TRAILING, TMRD_TIMING_B1_LEADING, 
  *   TMRD_TIMING_B1_TRAILING.
  * @param  Timing: 32bits timing value,the data range is 0x02~0x10000.
  * @retval None
  */
void TMRD_ChangeTiming(TSB_TD_TypeDef * TDx, uint8_t TimingType, uint32_t Timing)
{
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_TIMING_TYPE(TimingType));
    assert_param(IS_TMRD_TIMING_RANGE(Timing));

    /* Set TDxRGm */
    switch (TimingType) {
    case TMRD_TIMING_TD0_CYCLE:
        TSB_TD0->RG0 = Timing - 1U;
        break;
    case TMRD_TIMING_A0_LEADING:
        TSB_TD0->RG1 = Timing;
        break;
    case TMRD_TIMING_A0_TRAILING:
        TSB_TD0->RG2 &= RG24_TIMING_MASK;
        TSB_TD0->RG2 |= ((uint32_t) Timing << 4U);
        break;
    case TMRD_TIMING_A1_LEADING:
        TSB_TD0->RG3 = Timing;
        break;
    case TMRD_TIMING_A1_TRAILING:
        TSB_TD0->RG4 &= RG24_TIMING_MASK;
        TSB_TD0->RG4 |= ((uint32_t) Timing << 4U);
        break;
    case TMRD_TIMING_PHASE_SHIFT:
        TSB_TD0->RG5 = Timing;
        break;
    case TMRD_TIMING_TD1_CYCLE:
        TSB_TD1->RG0 = Timing - 1U;
        break;
    case TMRD_TIMING_B0_LEADING:
        TSB_TD1->RG1 = Timing;
        break;
    case TMRD_TIMING_B0_TRAILING:
        TSB_TD1->RG2 &= RG24_TIMING_MASK;
        TSB_TD1->RG2 |= ((uint32_t) Timing << 4U);
        break;
    case TMRD_TIMING_B1_LEADING:
        TSB_TD1->RG3 = Timing;
        break;
    case TMRD_TIMING_B1_TRAILING:
        TSB_TD1->RG4 &= RG24_TIMING_MASK;
        TSB_TD1->RG4 |= ((uint32_t) Timing << 4U);
        break;
    default:
        /* Do nothing */
        break;
    }

}

/**
  * @brief  Get the specified TMRD timing value. 
  * @param  TDx: Select the TMRD unit.
  *   This parameter can be one of the following values:
  *   TSB_TD.
  * @param  TimingType: Timing parameters type.
  *   This parameter can be one of the following values:
  *   TMRD_TIMING_TD0_CYCLE, TMRD_TIMING_A0_LEADING, 
  *   TMRD_TIMING_A0_TRAILING, TMRD_TIMING_A1_LEADING, 
  *   TMRD_TIMING_A1_TRAILING, TMRD_TIMING_PHASE_SHIFT, 
  *   TMRD_TIMING_TD1_CYCLE, TMRD_TIMING_B0_LEADING, 
  *   TMRD_TIMING_B0_TRAILING, TMRD_TIMING_B1_LEADING, 
  *   TMRD_TIMING_B1_TRAILING.
  * @retval Specified TMRD compare register timing.
  */
uint16_t TMRD_GetTiming(TSB_TD_TypeDef * TDx, uint8_t TimingType)
{
    uint16_t retval = 0U;
    /* Check the parameters */
    assert_param(IS_TMRD_ALL_UNIT(TDx));
    assert_param(IS_TMRD_TIMING_TYPE(TimingType));

    /* Get TDxRGm */
    switch (TimingType) {
    case TMRD_TIMING_TD0_CYCLE:
        retval = (uint16_t) TSB_TD0->CP0;
        break;
    case TMRD_TIMING_A0_LEADING:
        retval = (uint16_t) TSB_TD0->CP1;
        break;
    case TMRD_TIMING_A0_TRAILING:
        retval = (uint16_t) (TSB_TD0->CP2 >> 4U);
        break;
    case TMRD_TIMING_A1_LEADING:
        retval = (uint16_t) TSB_TD0->CP3;
        break;
    case TMRD_TIMING_A1_TRAILING:
        retval = (uint16_t) (TSB_TD0->CP4 >> 4U);
        break;
    case TMRD_TIMING_PHASE_SHIFT:
        retval = (uint16_t) TSB_TD0->CP5;
        break;
    case TMRD_TIMING_TD1_CYCLE:
        retval = (uint16_t) TSB_TD1->CP0;
        break;
    case TMRD_TIMING_B0_LEADING:
        retval = (uint16_t) TSB_TD1->CP1;
        break;
    case TMRD_TIMING_B0_TRAILING:
        retval = (uint16_t) (TSB_TD1->CP2 >> 4U);
        break;
    case TMRD_TIMING_B1_LEADING:
        retval = (uint16_t) TSB_TD1->CP3;
        break;
    case TMRD_TIMING_B1_TRAILING:
        retval = (uint16_t) (TSB_TD1->CP4 >> 4U);
        break;
    default:
        /* Do nothing */
        break;
    }

    return retval;
}



/** @} */
/* End of group TMRD_Exported_Functions */

/** @} */
/* End of group TMRD */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_TMRD_H) */
