/** 
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of TMRD interlock PPG demo for the TOSHIBA
 *          'TMPM440' Device 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "tmpm440_tmrd.h"
#include "tmpm440_gpio.h"
#include "tmpm440_cg.h"

/*
 * Cycle is 500us:
 * fosc = 10MHz(external high-speed oscillator)
 * fc = 10 * fosc = 100MHz(in this demo, PLL multiplying value is set to 10 multiplying)
 * ftmrd = fc / 2 = 50MHz(in this demo, ftmrd = fc / 2, please see CG_SetFtmrdSrc())
 * So the value is 500us*ftmrd = 25000 = 0x61A8U
 *
 * Duty is 50%:
 * Set LeadingTiming0 to 0x0000, then TrailingTiming0 is Cycle/2 = 0x30D4U(250us)
 * LeadingTiming1 and TrailingTiming1 are not used, so keep their value to 0x0000.
 *
 * Phase shift is 120 degree:
 * The phase shift can be calculated by using the formula:
 *
 * Fai = 360 degree*(PhaseShiftTiming/(Cycle+1))
 *
 * so in this demo, PhaseShiftTiming should be set to Cycle/3 = 0x208DU(167us)
 */
#define TIME_CYCLE               ((uint16_t)0x61A8U)    /* 500us */
#define TIME_DUTY                ((uint16_t)0x30D4U)    /* 250us */
#define TIME_PHASE_SHIFT         ((uint16_t)0x208DU)    /* 500/3= 167us */

int main(void)
{
    TMRD_TimingTypeDef timestruct = { 0U };

    timestruct.Cycle = TIME_CYCLE;
    timestruct.TrailingTiming0 = TIME_DUTY;
    timestruct.PhaseShiftTiming = TIME_PHASE_SHIFT;

    /* TD0OUT0, TD0OUT1, TD1OUT0, TD1OUT1 port setting */
    GPIO_SetOutput(GPIO_PY, GPIO_BIT_4 | GPIO_BIT_5 | GPIO_BIT_6 | GPIO_BIT_7);
    GPIO_EnableFuncReg(GPIO_PY, GPIO_FUNC_REG_1, GPIO_BIT_4 | GPIO_BIT_5 | GPIO_BIT_6 | GPIO_BIT_7);

    /* TMRD CG setting */
    CG_SetFtmrdSrc(CG_TMRD_UNIT_A, CG_FTMRD_SRC_HALF_FC);
    CG_SetTMRDClk(CG_TMRD_UNIT_A, ENABLE);

    /* TMRD mode setting */
    TMRD_SetMode(TSB_TD, TMRD_MODE_INTERLOCK_PPG_3CH);

    /* TMRD IDLE mode */
    TMRD_SetRunStateInIdle(TSB_TD, TMRD_UNIT_CH_0, TMRD_RUN);
    TMRD_SetRunStateInIdle(TSB_TD, TMRD_UNIT_CH_1, TMRD_RUN);

    /* TMRD enable clock signal */
    TMRD_Enable(TSB_TD, TMRD_UNIT_CH_0);
    TMRD_Enable(TSB_TD, TMRD_UNIT_CH_1);

    /* Set the TMRD prescaler */
    TMRD_SetClkDivision(TSB_TD, TMRD_UNIT_CH_0, TMRD_CLK_DIV_1);

    /* Set the PPG output edge */
    TMRD_SetPPGInitLeadingEdge(TSB_TD, TMRD_PPG_CHANNEL_A0, TMRD_WAVE_EDGE_RISING);
    TMRD_SetPPGInitLeadingEdge(TSB_TD, TMRD_PPG_CHANNEL_B0, TMRD_WAVE_EDGE_RISING);

    /* Direct update the compare register */
    TMRD_SetCMPRegWritePath(TSB_TD, TMRD_UNIT_CH_0, TMRD_CMP_WRITE_DIRECT);
    TMRD_SetCMPRegWritePath(TSB_TD, TMRD_UNIT_CH_1, TMRD_CMP_WRITE_DIRECT);

    /* Set the value to timer register */
    TMRD_SetInitTiming(TSB_TD, TMRD_UNIT_CH_0, &timestruct);
    TMRD_SetInitTiming(TSB_TD, TMRD_UNIT_CH_1, &timestruct);

    /* Set the relation of phase A and B */
    TMRD_SetPhaseRelation(TSB_TD, TMRD_PHASE_DELAY_OR_SAME);

    /* Indirct update the compare register */
    TMRD_SetCMPRegWritePath(TSB_TD, TMRD_UNIT_CH_0, TMRD_CMP_WRITE_INDIRECT);
    TMRD_SetCMPRegWritePath(TSB_TD, TMRD_UNIT_CH_1, TMRD_CMP_WRITE_INDIRECT);

    /* Start to run TMRD */
    TMRD_SetRunState(TSB_TD, TMRD_UNIT_CH_0, TMRD_RUN);
    while (1) {
      /* Do nothing */
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
